"""
Test that the po command acts correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class PoVerbosityTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.cpp.
        self.line = line_number("main.m", "// Stop here")

    @add_test_categories(["objc"])
    def test(self):
        """Test that the po command acts correctly."""
        self.build()

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synthetic clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        """Test expr + formatters for good interoperability."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        self.runCmd("settings set target.prefer-dynamic-value no-dynamic-values")

        self.expect("expr -O -v -- foo", substrs=["(id) $", " = 0x", "1 = 2", "2 = 3;"])
        self.expect(
            "expr -O -vfull -- foo", substrs=["(id) $", " = 0x", "1 = 2", "2 = 3;"]
        )
        self.expect("expr -O -- foo", matching=False, substrs=["(id) $"])

        self.expect("expr -O -- 22", matching=False, substrs=["(int) $"])
        self.expect("expr -O -- 22", substrs=["22"])

        self.expect("expr -O -vfull -- 22", substrs=["(int) $", " = 22"])

        self.expect("expr -O -v -- 22", substrs=["(int) $", " = 22"])



"""
Test python scripted process in lldb
"""

import os, shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test import lldbtest

import dummy_scripted_process


class ScriptedProcesTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    def test_python_plugin_package(self):
        """Test that the lldb python module has a `plugins.scripted_process`
        package."""
        self.expect(
            "script import lldb.plugins",
            substrs=["ModuleNotFoundError"],
            matching=False,
        )

        self.expect("script dir(lldb.plugins)", substrs=["scripted_process"])

        self.expect(
            "script import lldb.plugins.scripted_process",
            substrs=["ModuleNotFoundError"],
            matching=False,
        )

        self.expect(
            "script dir(lldb.plugins.scripted_process)", substrs=["ScriptedProcess"]
        )

        self.expect(
            "script from lldb.plugins.scripted_process import ScriptedProcess",
            substrs=["ImportError"],
            matching=False,
        )

        self.expect("script dir(ScriptedProcess)", substrs=["launch"])

    def move_blueprint_to_dsym(self, blueprint_name):
        blueprint_origin_path = os.path.join(self.getSourceDir(), blueprint_name)
        dsym_bundle = self.getBuildArtifact("a.out.dSYM")
        blueprint_destination_path = os.path.join(
            dsym_bundle, "Contents", "Resources", "Python"
        )
        if not os.path.exists(blueprint_destination_path):
            os.mkdir(blueprint_destination_path)

        blueprint_destination_path = os.path.join(
            blueprint_destination_path, "a_out.py"
        )
        shutil.copy(blueprint_origin_path, blueprint_destination_path)

    # No dylib on Windows.
    @skipIfWindows
    def test_missing_methods_scripted_register_context(self):
        """Test that we only instanciate scripted processes if they implement
        all the required abstract methods."""
        self.build()

        os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"] = "1"

        def cleanup():
            del os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"]

        self.addTearDownHook(cleanup)

        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)
        log_file = self.getBuildArtifact("script.log")
        self.runCmd("log enable lldb script -f " + log_file)
        self.assertTrue(os.path.isfile(log_file))
        script_path = os.path.join(
            self.getSourceDir(), "missing_methods_scripted_process.py"
        )
        self.runCmd("command script import " + script_path)

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName(
            "missing_methods_scripted_process.MissingMethodsScriptedProcess"
        )
        error = lldb.SBError()

        process = target.Launch(launch_info, error)

        self.assertFailure(error)

        with open(log_file, "r") as f:
            log = f.read()

        self.assertIn(
            "Abstract method MissingMethodsScriptedProcess.read_memory_at_address not implemented",
            log,
        )
        self.assertIn(
            "Abstract method MissingMethodsScriptedProcess.is_alive not implemented",
            log,
        )
        self.assertIn(
            "Abstract method MissingMethodsScriptedProcess.get_scripted_thread_plugin not implemented",
            log,
        )

    @skipUnlessDarwin
    def test_invalid_scripted_register_context(self):
        """Test that we can launch an lldb scripted process with an invalid
        Scripted Thread, with invalid register context."""
        self.build()

        os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"] = "1"

        def cleanup():
            del os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"]

        self.addTearDownHook(cleanup)

        self.runCmd("settings set target.load-script-from-symbol-file true")
        self.move_blueprint_to_dsym("invalid_scripted_process.py")
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)
        log_file = self.getBuildArtifact("thread.log")
        self.runCmd("log enable lldb thread -f " + log_file)
        self.assertTrue(os.path.isfile(log_file))

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName("a_out.InvalidScriptedProcess")
        error = lldb.SBError()

        process = target.Launch(launch_info, error)

        self.assertSuccess(error)
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 666)
        self.assertEqual(process.GetNumThreads(), 0)

        impl = process.GetScriptedImplementation()
        self.assertTrue(impl)
        impl = process.GetScriptedImplementation()
        self.assertTrue(impl)
        impl = process.GetScriptedImplementation()
        self.assertTrue(impl)
        impl = process.GetScriptedImplementation()
        self.assertTrue(impl)

        addr = 0x500000000
        buff = process.ReadMemory(addr, 4, error)
        self.assertEqual(buff, None)
        self.assertTrue(error.Fail())
        self.assertEqual(error.GetCString(), "This is an invalid scripted process!")

        with open(log_file, "r") as f:
            log = f.read()

        self.assertIn("Failed to get scripted thread registers data.", log)

    @skipUnlessDarwin
    def test_scripted_process_and_scripted_thread(self):
        """Test that we can launch an lldb scripted process using the SBAPI,
        check its process ID, read string from memory, check scripted thread
        id, name stop reason and register context.
        """
        self.build()
        target_0 = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target_0, VALID_TARGET)

        os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"] = "1"

        def cleanup():
            del os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"]

        self.addTearDownHook(cleanup)

        scripted_process_example_relpath = "dummy_scripted_process.py"
        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), scripted_process_example_relpath)
        )

        self.runCmd(
            "target stop-hook add -k first -v 1 -k second -v 2 -P dummy_scripted_process.DummyStopHook"
        )

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName(
            "dummy_scripted_process.DummyScriptedProcess"
        )

        error = lldb.SBError()
        process_0 = target_0.Launch(launch_info, error)
        self.assertTrue(process_0 and process_0.IsValid(), PROCESS_IS_VALID)
        self.assertEqual(process_0.GetProcessID(), 42)
        self.assertEqual(process_0.GetNumThreads(), 1)

        py_impl = process_0.GetScriptedImplementation()
        self.assertTrue(py_impl)
        self.assertIsInstance(py_impl, dummy_scripted_process.DummyScriptedProcess)
        self.assertFalse(hasattr(py_impl, "my_super_secret_member"))
        py_impl.my_super_secret_member = 42
        self.assertTrue(hasattr(py_impl, "my_super_secret_member"))
        self.assertEqual(py_impl.my_super_secret_method(), 42)

        self.assertTrue(hasattr(py_impl, "handled_stop"))
        self.assertTrue(py_impl.handled_stop)

        # Try reading from target #0 process ...
        addr = 0x500000000
        message = "Hello, target 0"
        buff = process_0.ReadCStringFromMemory(addr, len(message) + 1, error)
        self.assertSuccess(error)
        self.assertEqual(buff, message)

        target_1 = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target_1, VALID_TARGET)

        # We still need to specify a PID when attaching even for scripted processes
        attach_info = lldb.SBAttachInfo(42)
        attach_info.SetProcessPluginName("ScriptedProcess")
        attach_info.SetScriptedProcessClassName(
            "dummy_scripted_process.DummyScriptedProcess"
        )

        error = lldb.SBError()
        process_1 = target_1.Attach(attach_info, error)
        self.assertTrue(process_1 and process_1.IsValid(), PROCESS_IS_VALID)
        self.assertEqual(process_1.GetProcessID(), 42)
        self.assertEqual(process_1.GetNumThreads(), 1)

        # ... then try reading from target #1 process ...
        message = "Hello, target 1"
        buff = process_1.ReadCStringFromMemory(addr, len(message) + 1, error)
        self.assertSuccess(error)
        self.assertEqual(buff, message)

        # ... now, reading again from target #0 process to make sure the call
        # gets dispatched to the right target.
        message = "Hello, target 0"
        buff = process_0.ReadCStringFromMemory(addr, len(message) + 1, error)
        self.assertSuccess(error)
        self.assertEqual(buff, message)

        # Let's write some memory.
        message = "Hello, world!"
        bytes_written = process_0.WriteMemoryAsCString(addr, message, error)
        self.assertSuccess(error)
        self.assertEqual(bytes_written, len(message) + 1)

        # ... and check if that memory was saved properly.
        buff = process_0.ReadCStringFromMemory(addr, len(message) + 1, error)
        self.assertSuccess(error)
        self.assertEqual(buff, message)

        thread = process_0.GetSelectedThread()
        self.assertTrue(thread, "Invalid thread.")
        self.assertEqual(thread.GetThreadID(), 0x19)
        self.assertEqual(thread.GetName(), "DummyScriptedThread.thread-1")
        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonTrace)

        self.assertGreater(thread.GetNumFrames(), 0)

        frame = thread.GetFrameAtIndex(0)
        GPRs = None
        register_set = frame.registers  # Returns an SBValueList.
        for regs in register_set:
            if "general purpose" in regs.name.lower():
                GPRs = regs
                break

        self.assertTrue(GPRs, "Invalid General Purpose Registers Set")
        self.assertGreater(GPRs.GetNumChildren(), 0)
        for idx, reg in enumerate(GPRs, start=1):
            if idx > 21:
                break
            self.assertEqual(idx, int(reg.value, 16))

        self.assertTrue(frame.IsSynthetic(), "Frame is not synthetic")
        pc = frame.GetPCAddress().GetLoadAddress(target_0)
        self.assertEqual(pc, 0x0100001B00)

        # Regression test: ScriptedFrame.__init__ used GetThreadByIndexID(tid)
        # instead of GetThreadByID(tid). If the thread ID doesn't match any
        # thread index, the lookup returns an invalid SBThread. Downstream,
        # resolving the execution context from an SBFrame backed by this
        # invalid thread yields a null Process, and calling
        # ProcessModID::IsRunningExpression() on it crashes.
        tid = thread.GetThreadID()
        self.assertTrue(
            process_0.GetThreadByID(tid).IsValid(),
            "GetThreadByID(tid) should find the thread",
        )
        self.assertFalse(
            process_0.GetThreadByIndexID(tid).IsValid(),
            "GetThreadByIndexID(tid) should NOT find the thread "
            "(index ID != thread ID)",
        )

        # Construct a new ScriptedFrame with the SBThread (which carries the
        # real thread ID) and verify its thread member is valid. This exercises
        # the SBThread branch in ScriptedFrame.__init__ where thread.id is
        # used with GetThreadByID.
        post_launch_frame = dummy_scripted_process.DummyScriptedFrame(
            thread, lldb.SBStructuredData(), 42, "test_frame"
        )
        self.assertTrue(
            post_launch_frame.thread.IsValid(),
            "ScriptedFrame.thread should be valid after thread " "registration",
        )
        self.assertEqual(post_launch_frame.thread.GetThreadID(), tid)



"""
Test SBFunction::GetBaseName() and SBSymbol::GetBaseName() APIs.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GetBaseNameTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.main_source_file = lldb.SBFileSpec("main.cpp")

    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="https://github.com/llvm/llvm-project/issues/156861",
    )
    def test(self):
        """Test SBFunction.GetBaseName() and SBSymbol.GetBaseName()"""
        self.build()
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        frame0 = thread.GetFrameAtIndex(0)

        # Get both function and symbol
        function = frame0.GetFunction()
        symbol = frame0.GetSymbol()

        # Test consistency between function and symbol basename
        function_basename = function.GetBaseName()
        symbol_basename = symbol.GetBaseName()

        self.assertEqual(function_basename, "templateFunc")
        self.assertEqual(symbol_basename, "templateFunc")



#!/usr/bin/env python3
"""
Usage: convert-lldb-header-to-rpc-header.py <path/to/input-header.h> <path/to/output-header.h>

This scripts takes common LLDB headers (such as lldb-defines.h) and replaces references to LLDB
with those for RPC. This happens for:
- namespace definitions
- namespace usage
- version string macros
- ifdef/ifndef lines
"""

import argparse
import os
import re


INCLUDES_TO_REMOVE_REGEX = re.compile(
    r'#include "lldb/lldb-forward.h"|#include "lldb/lldb-versioning.h"'
)
LLDB_GUARD_REGEX = re.compile(r"(?P<guard_type>#.+)LLDB_LLDB_\s*", re.M)
LLDB_API_GUARD_REGEX = re.compile(r"(?P<guard_type>#.+)LLDB_API_\s*", re.M)
LLDB_VERSION_REGEX = re.compile(r"#define LLDB_VERSION", re.M)
LLDB_REVISION_REGEX = re.compile(r"#define LLDB_REVISION", re.M)
LLDB_VERSION_STRING_REGEX = re.compile(r"#define LLDB_VERSION_STRING", re.M)
LLDB_LOCAL_INCLUDE_REGEX = re.compile(r'#include "lldb/lldb-\s*', re.M)
LLDB_NAMESPACE_DEFINITION_REGEX = re.compile(
    r"(?P<comment_marker>//\s*){,1}namespace lldb\s{1}", re.M
)
LLDB_NAMESPACE_REGEX = re.compile(r"lldb::\s*", re.M)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("input")
    parser.add_argument("output")
    args = parser.parse_args()
    input_path = str(args.input)
    output_path = str(args.output)
    with open(input_path, "r") as input_file:
        lines = input_file.readlines()
        file_buffer = "".join(lines)

    with open(output_path, "w") as output_file:
        # NOTE: We do not use lldb-forward.h or lldb-versioning.h in RPC, so remove
        # all includes that are found for these files.
        file_buffer = re.sub(INCLUDES_TO_REMOVE_REGEX, r"", file_buffer)

        # For lldb-rpc-defines.h, replace the ifndef LLDB_LLDB_ portion with LLDB_RPC_ as we're not
        # using LLDB private definitions in RPC.
        lldb_guard_matches = LLDB_GUARD_REGEX.finditer(file_buffer)
        for match in lldb_guard_matches:
            file_buffer = re.sub(
                match.group(),
                r"{0}LLDB_RPC_".format(match.group("guard_type")),
                file_buffer,
            )

        # Similarly to lldb-rpc-defines.h, replace the ifndef for LLDB_API in SBDefines.h to LLDB_RPC_API_ for the same reason.
        lldb_api_guard_matches = LLDB_API_GUARD_REGEX.finditer(file_buffer)
        for match in lldb_api_guard_matches:
            file_buffer = re.sub(
                match.group(),
                r"{0}LLDB_RPC_API_".format(match.group("guard_type")),
                file_buffer,
            )

        # Replace the references for the macros that define the versioning strings in
        # lldb-rpc-defines.h.
        # NOTE: Here we assume that the versioning info has already been uncommented and
        # populated from the original lldb-defines.h.
        file_buffer = re.sub(
            LLDB_VERSION_REGEX, r"#define LLDB_RPC_VERSION", file_buffer
        )
        file_buffer = re.sub(
            LLDB_REVISION_REGEX, r"#define LLDB_RPC_REVISION", file_buffer
        )
        file_buffer = re.sub(
            LLDB_VERSION_STRING_REGEX, r"#define LLDB_RPC_VERSION_STRING", file_buffer
        )

        # For local #includes
        file_buffer = re.sub(
            LLDB_LOCAL_INCLUDE_REGEX, r'#include "lldb-rpc-', file_buffer
        )

        # Rename the lldb namespace definition to lldb-rpc.
        lldb_rpc_namespace_definition_matches = (
            LLDB_NAMESPACE_DEFINITION_REGEX.finditer(file_buffer)
        )
        for match in lldb_rpc_namespace_definition_matches:
            comment_marker = (
                match.group("comment_marker") if match.group("comment_marker") else ""
            )
            file_buffer = re.sub(
                match.group(),
                r"{0}namespace lldb_rpc ".format(comment_marker),
                file_buffer,
            )

        # Rename the lldb namespace definition to lldb-rpc.
        file_buffer = re.sub(LLDB_NAMESPACE_REGEX, r"lldb_rpc::", file_buffer)

        output_file.write(file_buffer)


if __name__ == "__main__":
    main()



"""Test that the Objective-C syntax for dictionary/array literals and indexing works"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCNewSyntaxTest import ObjCNewSyntaxTest


class ObjCNewSyntaxTestCaseLiteral(ObjCNewSyntaxTest):
    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_char_literal(self):
        self.runToBreakpoint()

        self.expect(
            "expr --object-description -- @'a'",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=[str(ord("a"))],
        )

    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_integer_literals(self):
        self.runToBreakpoint()

        self.expect(
            "expr --object-description -- @1",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1"],
        )

        self.expect(
            "expr --object-description -- @1l",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1"],
        )

        self.expect(
            "expr --object-description -- @1ul",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1"],
        )

        self.expect(
            "expr --object-description -- @1ll",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1"],
        )

        self.expect(
            "expr --object-description -- @1ull",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1"],
        )

    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_float_literal(self):
        self.runToBreakpoint()

        self.runCmd("settings set target.prefer-dynamic-value no-dynamic-values")

        self.expect(
            "expr -- @123.45",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["NSNumber", "123.45"],
        )

    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_expressions_in_literals(self):
        self.runToBreakpoint()

        self.runCmd("settings set target.prefer-dynamic-value no-dynamic-values")

        self.expect(
            "expr --object-description -- @( 1 + 3 )",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["4"],
        )
        self.expect(
            'expr -- @((char*)"Hello world" + 6)',
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["NSString", "world"],
        )



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *

import gdbremote_testcase


class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
    @skipIfWindows  # no SIGSEGV support
    @add_test_categories(["llgs"])
    def test_run(self):
        self.build()
        self.set_inferior_startup_launch()
        thread_num = 3
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["thread:segfault"] + thread_num * ["thread:new"]
        )
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

        segv_signo = lldbutil.get_signal_number("SIGSEGV")
        all_threads = set()
        all_segv_threads = []

        # we should get segfaults from all the threads
        for segv_no in range(thread_num):
            # first wait for the notification event
            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    {
                        "direction": "send",
                        "regex": r"^%Stop:(T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);)",
                        "capture": {1: "packet", 2: "signo", 3: "thread_id"},
                    },
                ],
                True,
            )
            m = self.expect_gdbremote_sequence()
            del m["O_content"]
            threads = [m]

            # then we may get events for the remaining threads
            # (but note that not all threads may have been started yet)
            while True:
                self.reset_test_sequence()
                self.test_sequence.add_log_lines(
                    [
                        "read packet: $vStopped#00",
                        {
                            "direction": "send",
                            "regex": r"^\$(OK|T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);)",
                            "capture": {1: "packet", 2: "signo", 3: "thread_id"},
                        },
                    ],
                    True,
                )
                m = self.expect_gdbremote_sequence()
                if m["packet"] == "OK":
                    break
                del m["O_content"]
                threads.append(m)

            segv_threads = []
            other_threads = []
            for t in threads:
                signo = int(t["signo"], 16)
                if signo == segv_signo:
                    segv_threads.append(t["thread_id"])
                else:
                    self.assertEqual(signo, 0)
                    other_threads.append(t["thread_id"])

            # verify that exactly one thread segfaulted
            self.assertEqual(len(segv_threads), 1)
            # we should get only one segv from every thread
            self.assertNotIn(segv_threads[0], all_segv_threads)
            all_segv_threads.extend(segv_threads)
            # segv_threads + other_threads should always be a superset
            # of all_threads, i.e. we should get states for all threads
            # already started
            self.assertFalse(all_threads.difference(other_threads + segv_threads))
            all_threads.update(other_threads + segv_threads)

            # verify that `?` returns the same result
            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    "read packet: $?#00",
                ],
                True,
            )
            threads_verify = []
            while True:
                self.test_sequence.add_log_lines(
                    [
                        {
                            "direction": "send",
                            "regex": r"^\$(OK|T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);)",
                            "capture": {1: "packet", 2: "signo", 3: "thread_id"},
                        },
                    ],
                    True,
                )
                m = self.expect_gdbremote_sequence()
                if m["packet"] == "OK":
                    break
                del m["O_content"]
                threads_verify.append(m)
                self.reset_test_sequence()
                self.test_sequence.add_log_lines(
                    [
                        "read packet: $vStopped#00",
                    ],
                    True,
                )

            self.assertEqual(threads, threads_verify)

            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    "read packet: $vCont;C{:02x}:{};c#00".format(
                        segv_signo, segv_threads[0]
                    ),
                    "send packet: $OK#00",
                ],
                True,
            )
            self.expect_gdbremote_sequence()

        # finally, verify that all threads have started
        self.assertEqual(len(all_threads), thread_num + 1)

    @add_test_categories(["llgs"])
    def test_vCtrlC(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(inferior_args=["thread:new"])
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                "read packet: $vCtrlC#00",
                "send packet: $OK#00",
                {
                    "direction": "send",
                    "regex": r"^%Stop:T",
                },
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @add_test_categories(["llgs"])
    def test_exit(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior()
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                "send packet: %Stop:W00#00",
                "read packet: $vStopped#00",
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @skipIfWindows  # no clue, the result makes zero sense
    @add_test_categories(["llgs"])
    def test_exit_query(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior()
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                "send packet: %Stop:W00#00",
                "read packet: $?#00",
                "send packet: $W00#00",
                "read packet: $vStopped#00",
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    def multiple_resume_test(self, second_command):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(inferior_args=["sleep:15"])
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                "read packet: ${}#00".format(second_command),
                "send packet: $E37#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @add_test_categories(["llgs"])
    def test_multiple_C_continue_with_signal(self):
        self.multiple_resume_test("C05")

    @add_test_categories(["llgs"])
    def test_multiple_c_continue_with_addr(self):
        self.multiple_resume_test("c")

    @add_test_categories(["llgs"])
    def test_multiple_s_single_step_with_addr(self):
        self.multiple_resume_test("s")

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_multiple_vCont(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["thread:new", "stop", "sleep:15"]
        )
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                {
                    "direction": "send",
                    "regex": r"^%Stop:T[0-9a-fA-F]{2}thread:([0-9a-fA-F]+);",
                    "capture": {1: "tid1"},
                },
                "read packet: $vStopped#63",
                {
                    "direction": "send",
                    "regex": r"^[$]T[0-9a-fA-F]{2}thread:([0-9a-fA-F]+);",
                    "capture": {1: "tid2"},
                },
                "read packet: $vStopped#63",
                "send packet: $OK#00",
            ],
            True,
        )
        ret = self.expect_gdbremote_sequence()

        self.reset_test_sequence()
        self.test_sequence.add_log_lines(
            [
                "read packet: $vCont;c:{}#00".format(ret["tid1"]),
                "send packet: $OK#00",
                "read packet: $vCont;c:{}#00".format(ret["tid2"]),
                "send packet: $E37#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @add_test_categories(["llgs"])
    @skipIfWindows  # Sometimes results in '$E37' instead of expected '$OK'
    def test_vCont_then_stop(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(inferior_args=["sleep:15"])
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                "read packet: $vCont;t#00",
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    def vCont_then_partial_stop_test(self, run_both):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["thread:new", "stop", "sleep:15"]
        )
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                {
                    "direction": "send",
                    "regex": r"^%Stop:T[0-9a-fA-F]{2}thread:([0-9a-fA-F]+);",
                    "capture": {1: "tid1"},
                },
                "read packet: $vStopped#63",
                {
                    "direction": "send",
                    "regex": r"^[$]T[0-9a-fA-F]{2}thread:([0-9a-fA-F]+);",
                    "capture": {1: "tid2"},
                },
                "read packet: $vStopped#63",
                "send packet: $OK#00",
            ],
            True,
        )
        ret = self.expect_gdbremote_sequence()

        self.reset_test_sequence()
        if run_both:
            self.test_sequence.add_log_lines(
                [
                    "read packet: $vCont;c#00",
                ],
                True,
            )
        else:
            self.test_sequence.add_log_lines(
                [
                    "read packet: $vCont;c:{}#00".format(ret["tid1"]),
                ],
                True,
            )
        self.test_sequence.add_log_lines(
            [
                "send packet: $OK#00",
                "read packet: $vCont;t:{}#00".format(ret["tid2"]),
                "send packet: $E03#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_vCont_then_partial_stop(self):
        self.vCont_then_partial_stop_test(False)

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_vCont_then_partial_stop_run_both(self):
        self.vCont_then_partial_stop_test(True)

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_stdio(self):
        self.build()
        self.set_inferior_startup_launch()
        # Since we can't easily ensure that lldb will send output in two parts,
        # just put a stop in the middle.  Since we don't clear vStdio,
        # the second message won't be delivered immediately.
        self.prep_debug_monitor_and_inferior(
            inferior_args=["message 1", "stop", "message 2"]
        )
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                {"direction": "send", "regex": r"^%Stop:T.*"},
                "read packet: $vStopped#00",
                "send packet: $OK#00",
                "read packet: $c#63",
                "send packet: $OK#00",
                "send packet: %Stop:W00#00",
            ],
            True,
        )
        ret = self.expect_gdbremote_sequence()

        # We know there will be at least two messages, but there may be more.
        # Loop until we have everything. The first message waiting for us in the
        # packet queue.
        count = 1
        output = self._server.get_raw_output_packet()
        while not (b"message 2\r\n" in output):
            self._server.send_packet(b"vStdio")
            output += self._server.get_raw_output_packet()
            count += 1
        self.assertGreaterEqual(count, 2)

        self.reset_test_sequence()
        self.test_sequence.add_log_lines(
            [
                "read packet: $vStdio#00",
                "send packet: $OK#00",
                "read packet: $vStopped#00",
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_stop_reason_while_running(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["thread:new", "thread:new", "stop", "sleep:15"]
        )
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                # stop is used to synchronize starting threads
                "read packet: $c#63",
                "send packet: $OK#00",
                {"direction": "send", "regex": "%Stop:T.*"},
                "read packet: $c#63",
                "send packet: $OK#00",
                "read packet: $?#00",
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_leave_nonstop(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["thread:new", "thread:new", "stop", "sleep:15"]
        )
        self.test_sequence.add_log_lines(
            [
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                # stop is used to synchronize starting threads
                "read packet: $c#63",
                "send packet: $OK#00",
                {"direction": "send", "regex": "%Stop:T.*"},
                "read packet: $c#63",
                "send packet: $OK#00",
                # verify that the threads are running now
                "read packet: $?#00",
                "send packet: $OK#00",
                "read packet: $QNonStop:0#00",
                "send packet: $OK#00",
                # we should issue some random request now to verify that the stub
                # did not send stop reasons -- we may verify whether notification
                # queue was cleared while at it
                "read packet: $vStopped#00",
                "send packet: $Eff#00",
                "read packet: $?#00",
                {"direction": "send", "regex": "[$]T.*"},
            ],
            True,
        )
        self.expect_gdbremote_sequence()



"""Test the integrity of the lldb public api directory containing SB*.h headers.

There should be nothing unwanted there and a simpe main.cpp which includes SB*.h
should compile and link with the LLDB framework."""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipIfNoSBHeaders
@skipIfRemote
@skipUnlessDarwin
class SBDirCheckerCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.cpp"
        self.generateSource(self.source)

    def test_sb_api_directory(self):
        """Test the SB API directory and make sure there's no unwanted stuff."""

        if not self.isAArch64() and self.getArchitecture() != "x86_64":
            self.skipTest("This test is only for LLDB.framework built 64-bit")

        exe_name = self.getBuildArtifact("a.out")
        self.buildDriver(self.source, exe_name)
        self.sanity_check_executable(exe_name)

    def sanity_check_executable(self, exe_name):
        """Sanity check executable compiled from the auto-generated program."""
        exe = self.getBuildArtifact(exe_name)
        self.runCmd("file %s" % exe, CURRENT_EXECUTABLE_SET)

        # This test uses a generated source file, so it's in the build directory.
        self.line_to_break = line_number(
            self.getBuildArtifact(self.source), "// Set breakpoint here."
        )

        lldbutil.run_break_set_by_file_and_line(
            self, self.source, self.line_to_break, num_expected_locations=-1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.runCmd("frame variable")



"""
Test multiword commands ('platform' in this case).
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class MultiwordCommandsTestCase(TestBase):
    @no_debug_info_test
    def test_ambiguous_subcommand(self):
        self.expect(
            "platform s",
            error=True,
            substrs=[
                "ambiguous command 'platform s'. Possible completions:",
                "\tselect\n",
                "\tsettings\n",
                "\tshell\n",
            ],
        )

    @no_debug_info_test
    def test_empty_subcommand(self):
        self.expect(
            'platform ""',
            error=True,
            substrs=["need to specify a non-empty subcommand"],
        )



"""
Test PDB enabled tests
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestBuildMethod(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def test(self):
        self.build()
        self.assertTrue(self.dbg.CreateTarget(self.getBuildArtifact()))
        if self.getDebugInfo() == "pdb":
            self.expect(
                "target modules dump symfile", patterns=["SymbolFile (native-)?pdb"]
            )



# RUN: %PYTHON %s | FileCheck %s
from functools import partialmethod

from mlir.ir import *
import mlir.dialects.arith as arith
import mlir.dialects.func as func
import mlir.dialects.llvm as llvm


def run(f):
    print("\nTEST:", f.__name__)
    f()


# CHECK-LABEL: TEST: testOverflowFlags
# Test mostly to repro and verify error addressed for Python bindings.
@run
def testOverflowFlags():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            a = arith.ConstantOp(value=42, result=IntegerType.get_signless(32))
            r = arith.AddIOp(a, a, overflowFlags=arith.IntegerOverflowFlags.nsw)
            # CHECK: arith.addi {{.*}}, {{.*}} overflow<nsw> : i32
            print(r)



"""
Test that we can compile expressions referring to
absent weak symbols from a dylib.
"""


import os
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestWeakSymbolsInExpressions(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    @skipIf(compiler="clang", compiler_version=["<", "19.0"])
    def test_weak_symbol_in_expr(self):
        """Tests that we can refer to weak symbols in expressions."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.do_test()

    def run_weak_var_check(self, weak_varname, present):
        # The expression will modify present_weak_int to signify which branch
        # was taken.  Set it to so we don't get confused by a previous run.
        value = self.target.FindFirstGlobalVariable("present_weak_int")
        value.SetValueFromCString("0")
        if present:
            correct_value = 10
        else:
            correct_value = 20

        # Note, I'm adding the "; 10" at the end of the expression to work around
        # the bug that expressions with no result currently return False for Success()...
        expr = (
            "if (&"
            + weak_varname
            + " != NULL) { present_weak_int = 10; } else { present_weak_int = 20;}; 10"
        )
        result = self.frame.EvaluateExpression(expr)
        self.assertSuccess(result.GetError(), "absent_weak_int expr failed")
        self.assertEqual(
            value.GetValueAsSigned(),
            correct_value,
            "Didn't change present_weak_int correctly.",
        )

    def do_test(self):
        hidden_dir = os.path.join(self.getBuildDir(), "hidden")
        hidden_dylib = os.path.join(hidden_dir, "libdylib.dylib")

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetWorkingDirectory(self.getBuildDir())
        launch_info.SetLaunchFlags(lldb.eLaunchFlagInheritTCCFromParent)

        (self.target, _, thread, _) = lldbutil.run_to_source_breakpoint(
            self,
            "Set a breakpoint here",
            self.main_source_file,
            launch_info=launch_info,
            extra_images=[hidden_dylib],
        )
        # First we have to import the Dylib module so we get the type info
        # for the weak symbol.  We need to add the source dir to the module
        # search paths, and then run @import to introduce it into the expression
        # context:
        self.dbg.HandleCommand(
            "settings set target.clang-module-search-paths " + self.getSourceDir()
        )

        self.frame = thread.frames[0]
        self.assertTrue(self.frame.IsValid(), "Got a good frame")
        options = lldb.SBExpressionOptions()
        options.SetLanguage(lldb.eLanguageTypeObjC)
        result = self.frame.EvaluateExpression("@import Dylib", options)

        # Now run an expression that references an absent weak symbol:
        self.run_weak_var_check("absent_weak_int", False)
        self.run_weak_var_check("absent_weak_function", False)

        # Make sure we can do the same thing with present weak symbols
        self.run_weak_var_check("present_weak_int", True)
        self.run_weak_var_check("present_weak_function", True)



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import expectedFailureWindows
from lldbsuite.test.lldbtest import line_number
import lldbdap_testcase


class TestDAP_launch_version(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests that "initialize" response contains the "version" string the same
    as the one returned by "version" command.
    """

    def test(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        source = "main.c"
        breakpoint_line = line_number(source, "// breakpoint 1")
        lines = [breakpoint_line]
        # Set breakpoint in the thread function so we can step the threads
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.continue_to_breakpoints(breakpoint_ids)

        version_eval_response = self.dap_server.request_evaluate(
            "`version", context="repl"
        )
        version_eval_output = version_eval_response["body"]["result"]

        version_string = self.dap_server.get_capability("$__lldb_version")
        self.assertEqual(
            version_eval_output.splitlines(),
            version_string.splitlines(),
            "version string does not match",
        )



"""
Test SBprocess and SBThread APIs with printing of the stack traces using lldbutil.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ThreadsStackTracesTestCase(TestBase):
    def test_stack_traces(self):
        """Test SBprocess and SBThread APIs with printing of the stack traces."""
        self.build()
        (_, process, _, _) = lldbutil.run_to_source_breakpoint(
            self, "// BREAK HERE", lldb.SBFileSpec("main.cpp")
        )
        stacktraces = lldbutil.print_stacktraces(process, string_buffer=True)
        self.expect(
            stacktraces,
            exe=False,
            substrs=["(int)x=4", "(int)y=6", "(int)x=3", "(int)argc=1"],
        )



"""
Tests that the import-std-module=fallback setting is only showing the error
diagnostics from the first parse attempt which isn't using a module.
This is supposed to prevent that a broken libc++ module renders failing
expressions useless as the original failing errors are suppressed by the
module build errors.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os


class TestCase(TestBase):
    # We only emulate a fake libc++ in this test and don't use the real libc++,
    # but we still add the libc++ category so that this test is only run in
    # test configurations where libc++ is actually supposed to be tested.
    @add_test_categories(["libc++"])
    @skipIfRemote
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        sysroot = os.path.join(os.getcwd(), "root")

        # Set the sysroot this test is using to provide a custom libc++.
        self.runCmd(
            "platform select --sysroot '" + sysroot + "' host", CURRENT_EXECUTABLE_SET
        )

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        # The expected error message when the fake libc++ module in this test
        # fails to build from within LLDB (as it contains invalid code).
        module_build_error_msg = "unknown type name 'random_token_to_fail_the_build'"

        # First force the std module to be imported. This should show the
        # module build error to the user.
        self.runCmd("settings set target.import-std-module true")
        self.expect(
            "expr (size_t)v.size()", substrs=[module_build_error_msg], error=True
        )

        # In the fallback mode the module build error should not be shown.
        self.runCmd("settings set target.import-std-module fallback")
        fallback_expr = "expr v ; error_to_trigger_fallback_mode"
        # First check for the actual expression error that should be displayed
        # and is useful for the user.
        self.expect(
            fallback_expr,
            substrs=["use of undeclared identifier 'error_to_trigger_fallback_mode'"],
            error=True,
        )
        # Test that the module build error is not displayed.
        self.expect(
            fallback_expr, substrs=[module_build_error_msg], matching=False, error=True
        )



import gdbremote_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestGdbRemoteAbort(gdbremote_testcase.GdbRemoteTestCaseBase):
    @skipIfWindows  # No signal is sent on Windows.
    # std::abort() on <= API 16 raises SIGSEGV - b.android.com/179836
    @expectedFailureAndroid(api_levels=list(range(16 + 1)))
    def test_inferior_abort_received_llgs(self):
        self.build()

        procs = self.prep_debug_monitor_and_inferior(inferior_args=["abort"])
        self.assertIsNotNone(procs)

        self.test_sequence.add_log_lines(
            [
                "read packet: $vCont;c#a8",
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2}).*#[0-9a-fA-F]{2}$",
                    "capture": {1: "hex_exit_code"},
                },
            ],
            True,
        )

        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        hex_exit_code = context.get("hex_exit_code")
        self.assertIsNotNone(hex_exit_code)
        self.assertEqual(int(hex_exit_code, 16), lldbutil.get_signal_number("SIGABRT"))



import lldb
import time
import unittest
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbreverse import ReverseTestBase
from lldbsuite.test import lldbutil


class TestReverseContinueWatchpoints(ReverseTestBase):
    @skipIfRemote
    # Watchpoints don't work in single-step mode
    @skipIf(macos_version=["<", "15.0"], archs=["arm64"])
    def test_reverse_continue_watchpoint(self):
        self.reverse_continue_watchpoint_internal(async_mode=False)

    @skipIfRemote
    # Watchpoints don't work in single-step mode
    @skipIf(macos_version=["<", "15.0"], archs=["arm64"])
    def test_reverse_continue_watchpoint_async(self):
        self.reverse_continue_watchpoint_internal(async_mode=True)

    def reverse_continue_watchpoint_internal(self, async_mode):
        target, process, initial_threads, watch_addr = self.setup_recording(async_mode)

        error = lldb.SBError()
        wp_opts = lldb.SBWatchpointOptions()
        wp_opts.SetWatchpointTypeWrite(lldb.eWatchpointWriteTypeOnModify)
        watchpoint = target.WatchpointCreateByAddress(watch_addr, 4, wp_opts, error)
        self.assertTrue(watchpoint)

        watch_var = target.EvaluateExpression("*g_watched_var_ptr")
        self.assertEqual(watch_var.GetValueAsSigned(-1), 2)

        # Reverse-continue to the function "trigger_watchpoint".
        status = process.ContinueInDirection(lldb.eRunReverse)
        self.assertSuccess(status)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        # We should stop at the point just before the location was modified.
        watch_var = target.EvaluateExpression("*g_watched_var_ptr")
        self.assertEqual(watch_var.GetValueAsSigned(-1), 1)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "trigger_watchpoint", "stop reason = watchpoint 1"],
        )

        # Stepping forward one instruction should change the value of the variable.
        initial_threads[0].StepInstruction(False)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        watch_var = target.EvaluateExpression("*g_watched_var_ptr")
        self.assertEqual(watch_var.GetValueAsSigned(-1), 2)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "trigger_watchpoint", "stop reason = watchpoint 1"],
        )

    @skipIfRemote
    # Watchpoints don't work in single-step mode
    @skipIf(macos_version=["<", "15.0"], archs=["arm64"])
    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/138084",
    )
    def test_reverse_continue_skip_watchpoint(self):
        self.reverse_continue_skip_watchpoint_internal(async_mode=False)

    @skipIfRemote
    # Watchpoints don't work in single-step mode
    @skipIf(macos_version=["<", "15.0"], archs=["arm64"])
    @skipIf(
        oslist=["windows"],
        archs=["x86_64"],
        bugnumber="github.com/llvm/llvm-project/issues/138084",
    )
    def test_reverse_continue_skip_watchpoint_async(self):
        self.reverse_continue_skip_watchpoint_internal(async_mode=True)

    def reverse_continue_skip_watchpoint_internal(self, async_mode):
        target, process, initial_threads, watch_addr = self.setup_recording(async_mode)

        # Reverse-continue over a watchpoint whose condition is false
        # (via function call).
        # This tests that we continue in the correct direction after hitting
        # the watchpoint.
        error = lldb.SBError()
        wp_opts = lldb.SBWatchpointOptions()
        wp_opts.SetWatchpointTypeWrite(lldb.eWatchpointWriteTypeOnModify)
        watchpoint = target.WatchpointCreateByAddress(watch_addr, 4, wp_opts, error)
        self.assertTrue(watchpoint)
        watchpoint.SetCondition("false_condition()")
        status = process.ContinueInDirection(lldb.eRunReverse)
        self.expect_async_state_changes(
            async_mode, process, [lldb.eStateRunning, lldb.eStateStopped]
        )
        self.assertSuccess(status)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_HISTORY_BOUNDARY,
            substrs=["stopped", "stop reason = history boundary"],
        )

    def setup_recording(self, async_mode):
        """
        Record execution of code between "start_recording" and "stop_recording" breakpoints.

        Returns with the target stopped at "stop_recording", with recording disabled,
        ready to reverse-execute.
        """
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        process = self.connect(target)

        # Record execution from the start of the function "start_recording"
        # to the start of the function "stop_recording". We want to keep the
        # interval that we record as small as possible to minimize the run-time
        # of our single-stepping recorder.
        start_recording_bkpt = target.BreakpointCreateByName("start_recording", None)
        self.assertTrue(start_recording_bkpt.GetNumLocations() > 0)
        initial_threads = lldbutil.continue_to_breakpoint(process, start_recording_bkpt)
        self.assertEqual(len(initial_threads), 1)
        target.BreakpointDelete(start_recording_bkpt.GetID())

        frame0 = initial_threads[0].GetFrameAtIndex(0)
        watched_var_ptr = frame0.FindValue(
            "g_watched_var_ptr", lldb.eValueTypeVariableGlobal
        )
        watch_addr = watched_var_ptr.GetValueAsUnsigned(0)
        self.assertTrue(watch_addr > 0)

        self.start_recording()
        stop_recording_bkpt = target.BreakpointCreateByName("stop_recording", None)
        self.assertTrue(stop_recording_bkpt.GetNumLocations() > 0)
        lldbutil.continue_to_breakpoint(process, stop_recording_bkpt)
        target.BreakpointDelete(stop_recording_bkpt.GetID())
        self.stop_recording()

        self.dbg.SetAsync(async_mode)
        self.expect_async_state_changes(async_mode, process, [lldb.eStateStopped])

        return target, process, initial_threads, watch_addr

    def expect_async_state_changes(self, async_mode, process, states):
        if not async_mode:
            return
        listener = self.dbg.GetListener()
        lldbutil.expect_state_changes(self, listener, process, states)



#!/usr/bin/env python3

"""
Usage: <path/to/input-directory> <path/to/output-directory>

This script is used when building LLDB.framework or LLDBRPC.framework. For each framework, local includes are converted to their respective framework includes.

This script is used in 2 ways:
1. It is used on header files that are copied into LLDB.framework. For these files, local LLDB includes are converted into framework includes, e.g. #include "lldb/API/SBDefines.h" -> #include <LLDB/SBDefines.h>.

2. It is used on header files for LLDBRPC.framework. For these files, includes of RPC common files will be converted to framework includes, e.g. #include <lldb-rpc/common/RPCCommon.h> -> #include <LLDBRPC/RPCCommon.h>. It will also change local includes to framework includes, e.g. #include "SBAddress.h" -> #include <LLDBRPC/SBAddress.h>
"""

import argparse
import os
import re
import shutil
import subprocess
import sys

# Main header regexes
INCLUDE_FILENAME_REGEX = re.compile(
    r'#include "lldb/(API/)?(?P<include_filename>.*){0,1}"'
)

# RPC header regexes
RPC_COMMON_REGEX = re.compile(r"#include <lldb-rpc/common/(?P<include_filename>.*)>")
RPC_INCLUDE_FILENAME_REGEX = re.compile(r'#include "(?P<include_filename>.*)"')


def modify_rpc_includes(input_file_path, output_file_path):
    with open(input_file_path, "r") as input_file:
        lines = input_file.readlines()
        file_buffer = "".join(lines)
        with open(output_file_path, "w") as output_file:
            # Local includes must be changed to RPC framework level includes.
            # e.g. #include "SBDefines.h" -> #include <LLDBRPC/SBDefines.h>
            # Also, RPC common code includes must change to RPC framework level includes.
            # e.g. #include "lldb-rpc/common/RPCPublic.h" -> #include <LLDBRPC/RPCPublic.h>
            rpc_common_matches = RPC_COMMON_REGEX.finditer(file_buffer)
            rpc_include_filename_matches = RPC_INCLUDE_FILENAME_REGEX.finditer(
                file_buffer
            )
            for match in rpc_common_matches:
                file_buffer = re.sub(
                    match.group(),
                    r"#include <LLDBRPC/" + match.group("include_filename") + ">",
                    file_buffer,
                )
            for match in rpc_include_filename_matches:
                file_buffer = re.sub(
                    match.group(),
                    r"#include <LLDBRPC/" + match.group("include_filename") + ">",
                    file_buffer,
                )
            output_file.write(file_buffer)


def modify_main_includes(input_file_path, output_file_path):
    with open(input_file_path, "r") as input_file:
        lines = input_file.readlines()
        file_buffer = "".join(lines)
        with open(output_file_path, "w") as output_file:
            # Local includes must be changed to framework level includes.
            # e.g. #include "lldb/API/SBDefines.h" -> #include <LLDB/SBDefines.h>
            regex_matches = INCLUDE_FILENAME_REGEX.finditer(file_buffer)
            for match in regex_matches:
                file_buffer = re.sub(
                    match.group(),
                    r"#include <LLDB/" + match.group("include_filename") + ">",
                    file_buffer,
                )
            output_file.write(file_buffer)


def remove_guards(output_file_path, unifdef_path, unifdef_guards):
    # The unifdef path should be passed in from CMake. If it wasn't there in CMake or is incorrect,
    # find it using shutil. If shutil can't find it, then exit.
    if not shutil.which(unifdef_path):
        unifdef_path = shutil.which("unifdef")
    if not unifdef_path:
        print(
            "Unable to find unifdef executable. Guards will not be removed from input files. Exiting..."
        )
        sys.exit()

    subprocess_command = (
        [unifdef_path, "-o", output_file_path] + unifdef_guards + [output_file_path]
    )
    subprocess.run(subprocess_command)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--framework", choices=["lldb_main", "lldb_rpc"])
    parser.add_argument("-i", "--input_file")
    parser.add_argument("-o", "--output_file")
    parser.add_argument("-p", "--unifdef_path")
    parser.add_argument(
        "--unifdef_guards",
        nargs="+",
        type=str,
        help="Guards to be removed with unifdef. These must be specified in the same way as they would be when passed directly into unifdef.",
    )
    args = parser.parse_args()
    input_file_path = str(args.input_file)
    output_file_path = str(args.output_file)
    framework_version = args.framework
    unifdef_path = str(args.unifdef_path)
    # Prepend dashes to the list of guards passed in from the command line.
    # unifdef takes the guards to remove as arguments in their own right (e.g. -USWIG)
    # but passing them in with dashes for this script causes argparse to think that they're
    # arguments in and of themself, so they need to passed in without dashes.
    if args.unifdef_guards:
        unifdef_guards = ["-U" + guard for guard in args.unifdef_guards]

    # Create the framework's header dir if it doesn't already exist
    try:
        os.makedirs(os.path.dirname(output_file_path))
    except FileExistsError:
        pass

    if framework_version == "lldb_main":
        modify_main_includes(input_file_path, output_file_path)
    if framework_version == "lldb_rpc":
        modify_rpc_includes(input_file_path, output_file_path)
    # After the incldues have been modified, run unifdef on the headers to remove any guards
    # specified at the command line.
    if args.unifdef_guards:
        remove_guards(output_file_path, unifdef_path, unifdef_guards)


if __name__ == "__main__":
    main()



"""
Test that 'stty -a' displays the same output before and after running the lldb command.
"""

import lldb
import io
import sys
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestSTTYBeforeAndAfter(TestBase):
    @classmethod
    def classCleanup(cls):
        """Cleanup the test byproducts."""
        cls.RemoveTempFile("child_send1.txt")
        cls.RemoveTempFile("child_read1.txt")
        cls.RemoveTempFile("child_send2.txt")
        cls.RemoveTempFile("child_read2.txt")

    @skipIf(macos_version=["<", "14.0"], asan=True)
    @add_test_categories(["pexpect"])
    @no_debug_info_test
    def test_stty_dash_a_before_and_afetr_invoking_lldb_command(self):
        """Test that 'stty -a' displays the same output before and after running the lldb command."""
        import pexpect

        if not which("expect"):
            self.skipTest("The 'expect' program cannot be located, skip the test")

        # The expect prompt.
        expect_prompt = "expect[0-9.]+> "
        # The default lldb prompt.
        lldb_prompt = "(lldb) "

        # So that the child gets torn down after the test.
        self.child = pexpect.spawn("expect", encoding="utf-8")
        child = self.child

        child.expect(expect_prompt)
        child.setecho(True)
        if self.TraceOn():
            child.logfile = sys.stdout

        if self.platformIsDarwin():
            child.sendline("set env(TERM) xterm")
        else:
            child.sendline("set env(TERM) vt100")
        child.expect(expect_prompt)
        child.sendline("puts $env(TERM)")
        child.expect(expect_prompt)

        # Turn on loggings for input/output to/from the child.
        child.logfile_send = child_send1 = io.StringIO()
        child.logfile_read = child_read1 = io.StringIO()
        child.sendline("stty -a")
        child.expect(expect_prompt)

        # Now that the stage1 logging is done, restore logfile to None to
        # stop further logging.
        child.logfile_send = None
        child.logfile_read = None

        # Invoke the lldb command.
        child.sendline(lldbtest_config.lldbExec)
        child.expect_exact(lldb_prompt)

        # Immediately quit.
        child.sendline("quit")
        child.expect(expect_prompt)

        child.logfile_send = child_send2 = io.StringIO()
        child.logfile_read = child_read2 = io.StringIO()
        child.sendline("stty -a")
        child.expect(expect_prompt)

        child.sendline("exit")

        # Now that the stage2 logging is done, restore logfile to None to
        # stop further logging.
        child.logfile_send = None
        child.logfile_read = None

        if self.TraceOn():
            print("\n\nContents of child_send1:")
            print(child_send1.getvalue())
            print("\n\nContents of child_read1:")
            print(child_read1.getvalue())
            print("\n\nContents of child_send2:")
            print(child_send2.getvalue())
            print("\n\nContents of child_read2:")
            print(child_read2.getvalue())

        stty_output1_lines = child_read1.getvalue().splitlines()
        stty_output2_lines = child_read2.getvalue().splitlines()
        zipped = list(zip(stty_output1_lines, stty_output2_lines))
        for tuple in zipped:
            if self.TraceOn():
                print("tuple->%s" % str(tuple))
            # Every line should compare equal until the first blank line.
            if len(tuple[0]) == 0:
                break
            self.assertEqual(tuple[0], tuple[1])



# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCDataFormatterTestCase import ObjCDataFormatterTestCase


class ObjCDataFormatterNSContainer(ObjCDataFormatterTestCase):
    SHARED_BUILD_TESTCASE = False

    def test_nscontainers_with_run_command(self):
        """Test formatters for  NS container classes."""
        self.appkit_tester_impl(self.nscontainers_data_formatter_commands, False)

    def nscontainers_data_formatter_commands(self):
        self.runCmd("settings set target.prefer-dynamic-value no-dynamic-values")

        self.expect(
            "frame variable newArray nsDictionary newDictionary nscfDictionary cfDictionaryRef newMutableDictionary copyDictionary newMutableDictionaryRef cfarray_ref mutable_array_ref",
            substrs=[
                "(NSArray *) newArray = ",
                ' @"50 elements"',
                "(NSDictionary *) nsDictionary = ",
                " 2 key/value pairs",
                "(NSDictionary *) newDictionary = ",
                " 12 key/value pairs",
                "(NSDictionary *) nscfDictionary = ",
                " 4 key/value pairs",
                "(CFDictionaryRef) cfDictionaryRef = ",
                " 2 key/value pairs",
                "(NSDictionary *) newMutableDictionary = ",
                " 21 key/value pairs",
                "(NSMutableDictionary *) copyDictionary = ",
                " 21 key/value pairs",
                "(CFMutableDictionaryRef) newMutableDictionaryRef = ",
                " 21 key/value pairs",
                "(CFArrayRef) cfarray_ref = ",
                ' @"3 elements"',
                "(CFMutableArrayRef) mutable_array_ref = ",
                ' @"11 elements"',
            ],
        )

        self.expect(
            "frame var -d run-target copyDictionary[10]", substrs=['@"bar9"', '@"foo"']
        )

        self.expect(
            "frame variable -d run-target *nscfDictionary",
            ordered=False,
            patterns=[
                r"\(__NSCFDictionary\) \*nscfDictionary =",
                'key = 0x.* @"foo"',
                'value = 0x.* @"foo"',
                'key = 0x.* @"bar"',
                'value = 0x.* @"bar"',
                'key = 0x.* @"baz"',
                'value = 0x.* @"baz"',
                'key = 0x.* @"quux"',
                'value = 0x.* @"quux"',
            ],
        )

        self.expect(
            "frame variable -d run-target *cfDictionaryRef",
            ordered=False,
            patterns=[
                r"\(const __CFDictionary\) \*cfDictionaryRef =",
                'key = 0x.* @"foo"',
                'value = 0x.* @"foo"',
                'key = 0x.* @"bar"',
                'value = 0x.* @"bar"',
            ],
        )

        self.expect(
            "frame var nscfSet cfSetRef",
            substrs=[
                "(NSSet *) nscfSet = ",
                "2 elements",
                "(CFSetRef) cfSetRef = ",
                "2 elements",
            ],
        )

        self.expect(
            "frame variable -d run-target *nscfSet",
            patterns=[
                r"\(__NSCFSet\) \*nscfSet =",
                r'\[0\] = 0x.* @".*"',
                r'\[1\] = 0x.* @".*"',
            ],
        )

        self.expect(
            "frame variable -d run-target *cfSetRef",
            patterns=[
                r"\(const __CFSet\) \*cfSetRef =",
                r'\[0\] = 0x.* @".*"',
                r'\[1\] = 0x.* @".*"',
            ],
        )

        self.expect(
            "frame variable iset1 iset2 imset",
            substrs=["4 indexes", "512 indexes", "10 indexes"],
        )

        self.expect(
            "frame variable binheap_ref",
            substrs=["(CFBinaryHeapRef) binheap_ref = ", '@"21 items"'],
        )

        self.expect(
            "expression -d run -- (NSArray*)[NSArray new]", substrs=['@"0 elements"']
        )



import lldb


def report_command(debugger, command, exe_ctx, result, internal_dict):
    result.AppendMessage(
        f'{lldb.num_module_inits} {lldb.num_target_inits} "{lldb.target_name}"'
    )
    result.SetStatus(lldb.eReturnStatusSuccessFinishResult)


def __lldb_init_module(debugger, internal_dict):
    # We only want to make one copy of the report command so it will be shared
    if "has_dsym_1" in __name__:
        # lldb is a convenient place to store our counters.
        lldb.num_module_inits = 0
        lldb.num_target_inits = 0
        lldb.target_name = "<unknown>"

        debugger.HandleCommand(
            f"command script add -o -f '{__name__}.report_command' report_command"
        )

    lldb.num_module_inits += 1


def __lldb_module_added_to_target(target, internal_dict):
    lldb.num_target_inits += 1
    target_name = target.executable.fullpath



import lldb
import json
import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


# On AArch64 systems, unused top bits of pointers can be used for other things.
@skipIf(archs=no_match(["aarch64", "arm64", "arm64e"]))
# Only run this test on systems where Top Byte Ignore is known to be enabled
# and widely available (FreeBSD has support but only since recently).
@skipUnlessPlatform(["linux"] + lldbplatformutil.getDarwinOSTriples())
class TestArmPointerMetadataStripping(TestBase):
    # Use extra_symbols.json as a template to add a new symbol whose address
    # contains non-zero high order bits set.
    def create_symbols_file(self):
        template_path = os.path.join(self.getSourceDir(), "extra_symbols.json")
        with open(template_path, "r") as f:
            symbols_data = json.load(f)

        target = self.dbg.GetSelectedTarget()
        symbols_data["triple"] = target.GetTriple()

        module = target.GetModuleAtIndex(0)
        symbols_data["uuid"] = module.GetUUIDString()

        json_filename = self.getBuildArtifact("extra_symbols.json")
        with open(json_filename, "w") as file:
            json.dump(symbols_data, file, indent=4)

        return json_filename

    def test(self):
        self.build()
        src = lldb.SBFileSpec("main.c")
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", src
        )

        symbols_file = self.create_symbols_file()
        self.runCmd(f"target module add {symbols_file}")

        # The high order bits should be stripped.
        self.expect_expr("get_high_bits(&myglobal_json)", result_value="0")

        # Mark all bits as used for addresses and ensure bits are no longer stripped.
        self.runCmd("settings set target.process.virtual-addressable-bits 64")
        self.expect_expr(
            "get_high_bits(&myglobal_json)", result_value=str(0x1200000000000000)
        )



"""
Test stop hooks
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbdap_testcase


class TestDAP_stop_hooks(lldbdap_testcase.DAPTestCaseBase):
    def test_stop_hooks_before_run(self):
        """
        Test that there is no race condition between lldb-dap and
        stop hooks executor
        """
        program = self.getBuildArtifact("a.out")
        preRunCommands = ["target stop-hook add -o help"]
        self.build_and_launch(program, preRunCommands=preRunCommands)
        breakpoint_ids = self.set_function_breakpoints(["main"])
        # This request hangs if the race happens, because, in that case, the
        # command interpreter is in synchronous mode while lldb-dap expects
        # it to be in asynchronous mode, so, the process doesn't send the stop
        # event to "lldb.Debugger" listener (which is monitored by lldb-dap).
        self.continue_to_breakpoints(breakpoint_ids)

        self.continue_to_exit()



# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCDataFormatterTestCase import ObjCDataFormatterTestCase


class ObjCDataFormatterNSPlain(ObjCDataFormatterTestCase):
    def test_plain_objc_with_run_command(self):
        """Test basic ObjC formatting behavior."""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.m", False)
        )

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd('type summary add --summary-string "${var%@}" MyClass')

        self.expect("frame variable object2", substrs=["MyOtherClass"])

        self.expect("frame variable *object2", substrs=["MyOtherClass"])

        # Now let's delete the 'MyClass' custom summary.
        self.runCmd("type summary delete MyClass")

        # The type format list should not show 'MyClass' at this point.
        self.expect("type summary list", matching=False, substrs=["MyClass"])

        self.runCmd('type summary add --summary-string "a test" MyClass')

        self.expect(
            "frame variable *object2",
            substrs=["*object2 =", "MyClass = a test", "backup = "],
        )

        self.expect("frame variable object2", matching=False, substrs=["a test"])

        self.expect("frame variable object", substrs=["a test"])

        self.expect("frame variable *object", substrs=["a test"])

        self.expect("frame variable myclass", substrs=["(Class) myclass = NSValue"])
        self.expect(
            "frame variable myclass2", substrs=["(Class) myclass2 = ", "NS", "String"]
        )
        self.expect("frame variable myclass3", substrs=["(Class) myclass3 = Molecule"])
        self.expect(
            "frame variable myclass4", substrs=["(Class) myclass4 = NSMutableArray"]
        )
        self.expect("frame variable myclass5", substrs=["(Class) myclass5 = nil"])



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.extras import types as T
from mlir.dialects import (
    arith,
    func,
    memref,
    scf,
    cf,
)
from mlir.passmanager import PassManager


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


# CHECK-LABEL: TEST: testSimpleForall
# CHECK: scf.forall (%[[IV0:.*]], %[[IV1:.*]]) in (4, 8) shared_outs(%[[BOUND_ARG:.*]] = %{{.*}}) -> (tensor<4x8xf32>)
# CHECK:   arith.addi %[[IV0]], %[[IV1]]
# CHECK:   scf.forall.in_parallel
@constructAndPrintInModule
def testSimpleForall():
    f32 = F32Type.get()
    tensor_type = RankedTensorType.get([4, 8], f32)

    @func.FuncOp.from_py_func(tensor_type)
    def forall_loop(tensor):
        loop = scf.ForallOp([0, 0], [4, 8], [1, 1], [tensor])
        with InsertionPoint(loop.body):
            i, j = loop.induction_variables
            arith.addi(i, j)
            loop.terminator()
        # The verifier will check that the regions have been created properly.
        assert loop.verify()


# CHECK-LABEL: TEST: testSimpleLoop
@constructAndPrintInModule
def testSimpleLoop():
    index_type = IndexType.get()

    @func.FuncOp.from_py_func(index_type, index_type, index_type)
    def simple_loop(lb, ub, step):
        loop = scf.ForOp(lb, ub, step, [lb, lb])
        with InsertionPoint(loop.body):
            scf.YieldOp(loop.inner_iter_args)
        return


# CHECK: func @simple_loop(%[[ARG0:.*]]: index, %[[ARG1:.*]]: index, %[[ARG2:.*]]: index)
# CHECK: scf.for %{{.*}} = %[[ARG0]] to %[[ARG1]] step %[[ARG2]]
# CHECK: iter_args(%[[I1:.*]] = %[[ARG0]], %[[I2:.*]] = %[[ARG0]])
# CHECK: scf.yield %[[I1]], %[[I2]]


# CHECK-LABEL: TEST: testInductionVar
@constructAndPrintInModule
def testInductionVar():
    index_type = IndexType.get()

    @func.FuncOp.from_py_func(index_type, index_type, index_type)
    def induction_var(lb, ub, step):
        loop = scf.ForOp(lb, ub, step, [lb])
        with InsertionPoint(loop.body):
            scf.YieldOp([loop.induction_variable])
        return


# CHECK: func @induction_var(%[[ARG0:.*]]: index, %[[ARG1:.*]]: index, %[[ARG2:.*]]: index)
# CHECK: scf.for %[[IV:.*]] = %[[ARG0]] to %[[ARG1]] step %[[ARG2]]
# CHECK: scf.yield %[[IV]]


# CHECK-LABEL: TEST: testForSugar
@constructAndPrintInModule
def testForSugar():
    index_type = IndexType.get()
    memref_t = MemRefType.get([10], index_type)
    range = scf.for_

    # CHECK:  func.func @range_loop_1(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    scf.for %[[VAL_4:.*]] = %[[VAL_0]] to %[[VAL_1]] step %[[VAL_2]] {
    # CHECK:      %[[VAL_5:.*]] = arith.addi %[[VAL_4]], %[[VAL_4]] : index
    # CHECK:      memref.store %[[VAL_5]], %[[VAL_3]]{{\[}}%[[VAL_4]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_1(lb, ub, step, memref_v):
        for i in range(lb, ub, step):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])

            scf.yield_([])

    # CHECK:  func.func @range_loop_2(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 10 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 1 : index
    # CHECK:    scf.for %[[VAL_6:.*]] = %[[VAL_0]] to %[[VAL_4]] step %[[VAL_5]] {
    # CHECK:      %[[VAL_7:.*]] = arith.addi %[[VAL_6]], %[[VAL_6]] : index
    # CHECK:      memref.store %[[VAL_7]], %[[VAL_3]]{{\[}}%[[VAL_6]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_2(lb, ub, step, memref_v):
        for i in range(lb, 10, 1):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])
            scf.yield_([])

    # CHECK:  func.func @range_loop_3(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 1 : index
    # CHECK:    scf.for %[[VAL_6:.*]] = %[[VAL_4]] to %[[VAL_1]] step %[[VAL_5]] {
    # CHECK:      %[[VAL_7:.*]] = arith.addi %[[VAL_6]], %[[VAL_6]] : index
    # CHECK:      memref.store %[[VAL_7]], %[[VAL_3]]{{\[}}%[[VAL_6]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_3(lb, ub, step, memref_v):
        for i in range(0, ub, 1):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])
            scf.yield_([])

    # CHECK:  func.func @range_loop_4(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 10 : index
    # CHECK:    scf.for %[[VAL_6:.*]] = %[[VAL_4]] to %[[VAL_5]] step %[[VAL_2]] {
    # CHECK:      %[[VAL_7:.*]] = arith.addi %[[VAL_6]], %[[VAL_6]] : index
    # CHECK:      memref.store %[[VAL_7]], %[[VAL_3]]{{\[}}%[[VAL_6]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_4(lb, ub, step, memref_v):
        for i in range(0, 10, step):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])
            scf.yield_([])

    # CHECK:  func.func @range_loop_5(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 10 : index
    # CHECK:    %[[VAL_6:.*]] = arith.constant 1 : index
    # CHECK:    scf.for %[[VAL_7:.*]] = %[[VAL_4]] to %[[VAL_5]] step %[[VAL_6]] {
    # CHECK:      %[[VAL_8:.*]] = arith.addi %[[VAL_7]], %[[VAL_7]] : index
    # CHECK:      memref.store %[[VAL_8]], %[[VAL_3]]{{\[}}%[[VAL_7]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_5(lb, ub, step, memref_v):
        for i in range(0, 10, 1):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])
            scf.yield_([])

    # CHECK:  func.func @range_loop_6(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 10 : index
    # CHECK:    %[[VAL_6:.*]] = arith.constant 1 : index
    # CHECK:    scf.for %[[VAL_7:.*]] = %[[VAL_4]] to %[[VAL_5]] step %[[VAL_6]] {
    # CHECK:      %[[VAL_8:.*]] = arith.addi %[[VAL_7]], %[[VAL_7]] : index
    # CHECK:      memref.store %[[VAL_8]], %[[VAL_3]]{{\[}}%[[VAL_7]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_6(lb, ub, step, memref_v):
        for i in range(0, 10):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])
            scf.yield_([])

    # CHECK:  func.func @range_loop_7(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 10 : index
    # CHECK:    %[[VAL_6:.*]] = arith.constant 1 : index
    # CHECK:    scf.for %[[VAL_7:.*]] = %[[VAL_4]] to %[[VAL_5]] step %[[VAL_6]] {
    # CHECK:      %[[VAL_8:.*]] = arith.addi %[[VAL_7]], %[[VAL_7]] : index
    # CHECK:      memref.store %[[VAL_8]], %[[VAL_3]]{{\[}}%[[VAL_7]]] : memref<10xindex>
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def range_loop_7(lb, ub, step, memref_v):
        for i in range(10):
            add = arith.addi(i, i)
            memref.store(add, memref_v, [i])
            scf.yield_([])

    # CHECK:  func.func @loop_yield_1(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[VAL_4:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_5:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_6:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_7:.*]] = arith.constant 100 : index
    # CHECK:    %[[VAL_8:.*]] = arith.constant 1 : index
    # CHECK:    %[[VAL_10:.*]] = scf.for %[[IV:.*]] = %[[VAL_6]] to %[[VAL_7]] step %[[VAL_8]] iter_args(%[[ITER:.*]] = %[[VAL_4]]) -> (index) {
    # CHECK:      %[[VAL_9:.*]] = arith.addi %[[ITER]], %[[IV]] : index
    # CHECK:      scf.yield %[[VAL_9]] : index
    # CHECK:    }
    # CHECK:    memref.store %[[VAL_10]], %[[VAL_3]]{{\[}}%[[VAL_5]]] : memref<10xindex>
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def loop_yield_1(lb, ub, step, memref_v):
        sum = arith.ConstantOp.create_index(0)
        c0 = arith.ConstantOp.create_index(0)
        for i, loc_sum, sum in scf.for_(0, 100, 1, [sum]):
            loc_sum = arith.addi(loc_sum, i)
            scf.yield_([loc_sum])
        memref.store(sum, memref_v, [c0])

    # CHECK:  func.func @loop_yield_2(%[[VAL_0:.*]]: index, %[[VAL_1:.*]]: index, %[[VAL_2:.*]]: index, %[[VAL_3:.*]]: memref<10xindex>) {
    # CHECK:    %[[c0:.*]] = arith.constant 0 : index
    # CHECK:    %[[c2:.*]] = arith.constant 2 : index
    # CHECK:    %[[REF1:.*]] = arith.constant 0 : index
    # CHECK:    %[[REF2:.*]] = arith.constant 1 : index
    # CHECK:    %[[VAL_6:.*]] = arith.constant 0 : index
    # CHECK:    %[[VAL_7:.*]] = arith.constant 100 : index
    # CHECK:    %[[VAL_8:.*]] = arith.constant 1 : index
    # CHECK:    %[[RES:.*]] = scf.for %[[IV:.*]] = %[[VAL_6]] to %[[VAL_7]] step %[[VAL_8]] iter_args(%[[ITER1:.*]] = %[[c0]], %[[ITER2:.*]] = %[[c2]]) -> (index, index) {
    # CHECK:      %[[VAL_9:.*]] = arith.addi %[[ITER1]], %[[IV]] : index
    # CHECK:      %[[VAL_10:.*]] = arith.addi %[[ITER2]], %[[IV]] : index
    # CHECK:      scf.yield %[[VAL_9]], %[[VAL_10]] : index, index
    # CHECK:    }
    # CHECK:    return
    # CHECK:  }
    @func.FuncOp.from_py_func(index_type, index_type, index_type, memref_t)
    def loop_yield_2(lb, ub, step, memref_v):
        sum1 = arith.ConstantOp.create_index(0)
        sum2 = arith.ConstantOp.create_index(2)
        c0 = arith.ConstantOp.create_index(0)
        c1 = arith.ConstantOp.create_index(1)
        for i, [loc_sum1, loc_sum2], [sum1, sum2] in scf.for_(0, 100, 1, [sum1, sum2]):
            loc_sum1 = arith.addi(loc_sum1, i)
            loc_sum2 = arith.addi(loc_sum2, i)
            scf.yield_([loc_sum1, loc_sum2])
        memref.store(sum1, memref_v, [c0])
        memref.store(sum2, memref_v, [c1])


@constructAndPrintInModule
def testOpsAsArguments():
    index_type = IndexType.get()
    callee = func.FuncOp("callee", ([], [index_type, index_type]), visibility="private")
    f = func.FuncOp("ops_as_arguments", ([], []))
    with InsertionPoint(f.add_entry_block()):
        lb = arith.ConstantOp.create_index(0)
        ub = arith.ConstantOp.create_index(42)
        step = arith.ConstantOp.create_index(2)
        iter_args = func.CallOp(callee, [])
        loop = scf.ForOp(lb, ub, step, iter_args)
        with InsertionPoint(loop.body):
            scf.YieldOp(loop.inner_iter_args)
        func.ReturnOp([])


# CHECK-LABEL: TEST: testOpsAsArguments
# CHECK: func private @callee() -> (index, index)
# CHECK: func @ops_as_arguments() {
# CHECK:   %[[LB:.*]] = arith.constant 0
# CHECK:   %[[UB:.*]] = arith.constant 42
# CHECK:   %[[STEP:.*]] = arith.constant 2
# CHECK:   %[[ARGS:.*]]:2 = call @callee()
# CHECK:   scf.for %arg0 = %c0 to %c42 step %c2
# CHECK:   iter_args(%{{.*}} = %[[ARGS]]#0, %{{.*}} = %[[ARGS]]#1)
# CHECK:     scf.yield %{{.*}}, %{{.*}}
# CHECK:   return


@constructAndPrintInModule
def testIfWithoutElse():
    bool = IntegerType.get_signless(1)
    i32 = IntegerType.get_signless(32)

    @func.FuncOp.from_py_func(bool)
    def simple_if(cond):
        if_op = scf.IfOp(cond)
        with InsertionPoint(if_op.then_block):
            one = arith.ConstantOp(i32, 1)
            add = arith.AddIOp(one, one)
            scf.YieldOp([])
        return


# CHECK: func @simple_if(%[[ARG0:.*]]: i1)
# CHECK: scf.if %[[ARG0:.*]]
# CHECK:   %[[ONE:.*]] = arith.constant 1
# CHECK:   %[[ADD:.*]] = arith.addi %[[ONE]], %[[ONE]]
# CHECK: return


@constructAndPrintInModule
def testNestedIf():
    bool = IntegerType.get_signless(1)
    i32 = IntegerType.get_signless(32)

    @func.FuncOp.from_py_func(bool, bool)
    def nested_if(b, c):
        if_op = scf.IfOp(b)
        with InsertionPoint(if_op.then_block) as ip:
            if_op = scf.IfOp(c, ip=ip)
            with InsertionPoint(if_op.then_block):
                one = arith.ConstantOp(i32, 1)
                add = arith.AddIOp(one, one)
                scf.YieldOp([])
            scf.YieldOp([])
        return


# CHECK: func @nested_if(%[[ARG0:.*]]: i1, %[[ARG1:.*]]: i1)
# CHECK: scf.if %[[ARG0:.*]]
# CHECK:   scf.if %[[ARG1:.*]]
# CHECK:     %[[ONE:.*]] = arith.constant 1
# CHECK:     %[[ADD:.*]] = arith.addi %[[ONE]], %[[ONE]]
# CHECK: return


@constructAndPrintInModule
def testIfWithElse():
    bool = IntegerType.get_signless(1)
    i32 = IntegerType.get_signless(32)

    @func.FuncOp.from_py_func(bool)
    def simple_if_else(cond):
        if_op = scf.IfOp(cond, [i32, i32], has_else=True)
        with InsertionPoint(if_op.then_block):
            x_true = arith.ConstantOp(i32, 0)
            y_true = arith.ConstantOp(i32, 1)
            scf.YieldOp([x_true, y_true])
        with InsertionPoint(if_op.else_block):
            x_false = arith.ConstantOp(i32, 2)
            y_false = arith.ConstantOp(i32, 3)
            scf.YieldOp([x_false, y_false])
        add = arith.AddIOp(if_op.results[0], if_op.results[1])
        return


# CHECK: func @simple_if_else(%[[ARG0:.*]]: i1)
# CHECK: %[[RET:.*]]:2 = scf.if %[[ARG0:.*]]
# CHECK:   %[[ZERO:.*]] = arith.constant 0
# CHECK:   %[[ONE:.*]] = arith.constant 1
# CHECK:   scf.yield %[[ZERO]], %[[ONE]]
# CHECK: } else {
# CHECK:   %[[TWO:.*]] = arith.constant 2
# CHECK:   %[[THREE:.*]] = arith.constant 3
# CHECK:   scf.yield %[[TWO]], %[[THREE]]
# CHECK: arith.addi %[[RET]]#0, %[[RET]]#1
# CHECK: return


@constructAndPrintInModule
def testIndexSwitch():
    i32 = T.i32()

    @func.FuncOp.from_py_func(T.index(), results=[i32])
    def index_switch(index):
        c1 = arith.constant(i32, 1)
        c0 = arith.constant(i32, 0)
        value = arith.constant(i32, 5)
        switch_op = scf.IndexSwitchOp([i32], index, range(3))

        assert switch_op.regions[0] == switch_op.default_region
        assert switch_op.regions[1] == switch_op.case_regions[0]
        assert switch_op.regions[1] == switch_op.case_region(0)
        assert len(switch_op.case_regions) == 3
        assert len(switch_op.regions) == 4

        with InsertionPoint(switch_op.default_block):
            cf.assert_(arith.constant(T.bool(), 0), "Whoops!")
            scf.yield_([c1])

        for i, block in enumerate(switch_op.case_blocks):
            with InsertionPoint(block):
                scf.yield_([arith.constant(i32, i)])

        func.return_([switch_op.results[0]])

    return index_switch


# CHECK-LABEL:   func.func @index_switch(
# CHECK-SAME:      %[[ARG0:.*]]: index) -> i32 {
# CHECK:           %[[CONSTANT_0:.*]] = arith.constant 1 : i32
# CHECK:           %[[CONSTANT_1:.*]] = arith.constant 0 : i32
# CHECK:           %[[CONSTANT_2:.*]] = arith.constant 5 : i32
# CHECK:           %[[INDEX_SWITCH_0:.*]] = scf.index_switch %[[ARG0]] -> i32
# CHECK:           case 0 {
# CHECK:             %[[CONSTANT_3:.*]] = arith.constant 0 : i32
# CHECK:             scf.yield %[[CONSTANT_3]] : i32
# CHECK:           }
# CHECK:           case 1 {
# CHECK:             %[[CONSTANT_4:.*]] = arith.constant 1 : i32
# CHECK:             scf.yield %[[CONSTANT_4]] : i32
# CHECK:           }
# CHECK:           case 2 {
# CHECK:             %[[CONSTANT_5:.*]] = arith.constant 2 : i32
# CHECK:             scf.yield %[[CONSTANT_5]] : i32
# CHECK:           }
# CHECK:           default {
# CHECK:             %[[CONSTANT_6:.*]] = arith.constant false
# CHECK:             cf.assert %[[CONSTANT_6]], "Whoops!"
# CHECK:             scf.yield %[[CONSTANT_0]] : i32
# CHECK:           }
# CHECK:           return %[[INDEX_SWITCH_0]] : i32
# CHECK:         }


@constructAndPrintInModule
def testIndexSwitchWithBodyBuilders():
    i32 = T.i32()

    @func.FuncOp.from_py_func(T.index(), results=[i32])
    def index_switch(index):
        c1 = arith.constant(i32, 1)
        c0 = arith.constant(i32, 0)
        value = arith.constant(i32, 5)

        def default_body_builder(switch_op):
            cf.assert_(arith.constant(T.bool(), 0), "Whoops!")
            scf.yield_([c1])

        def case_body_builder(switch_op, case_index: int, case_value: int):
            scf.yield_([arith.constant(i32, case_value)])

        result = scf.index_switch(
            results=[i32],
            arg=index,
            cases=range(3),
            case_body_builder=case_body_builder,
            default_body_builder=default_body_builder,
        )

        func.return_([result])

    return index_switch


# CHECK-LABEL:   func.func @index_switch(
# CHECK-SAME:      %[[ARG0:.*]]: index) -> i32 {
# CHECK:           %[[CONSTANT_0:.*]] = arith.constant 1 : i32
# CHECK:           %[[CONSTANT_1:.*]] = arith.constant 0 : i32
# CHECK:           %[[CONSTANT_2:.*]] = arith.constant 5 : i32
# CHECK:           %[[INDEX_SWITCH_0:.*]] = scf.index_switch %[[ARG0]] -> i32
# CHECK:           case 0 {
# CHECK:             %[[CONSTANT_3:.*]] = arith.constant 0 : i32
# CHECK:             scf.yield %[[CONSTANT_3]] : i32
# CHECK:           }
# CHECK:           case 1 {
# CHECK:             %[[CONSTANT_4:.*]] = arith.constant 1 : i32
# CHECK:             scf.yield %[[CONSTANT_4]] : i32
# CHECK:           }
# CHECK:           case 2 {
# CHECK:             %[[CONSTANT_5:.*]] = arith.constant 2 : i32
# CHECK:             scf.yield %[[CONSTANT_5]] : i32
# CHECK:           }
# CHECK:           default {
# CHECK:             %[[CONSTANT_6:.*]] = arith.constant false
# CHECK:             cf.assert %[[CONSTANT_6]], "Whoops!"
# CHECK:             scf.yield %[[CONSTANT_0]] : i32
# CHECK:           }
# CHECK:           return %[[INDEX_SWITCH_0]] : i32
# CHECK:         }



"""
debugserver used to block replying to the 'D' packet
till it had joined the profiling thread.  If the profiling interval
was too long, that would mean it would take longer than the packet
timeout to reply, and the detach would time out.  Make sure that doesn't
happen.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import os
import signal


class TestDetachVrsProfile(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    @skipIfRemote
    def test_profile_and_detach(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.do_profile_and_detach()

    def do_profile_and_detach(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()

        # First make sure we are getting async data.  Set a short interval, continue a bit and check:
        interp.HandleCommand(
            "process plugin packet send 'QSetEnableAsyncProfiling;enable:1;interval_usec:500000;scan_type=0x5;'",
            result,
        )
        self.assertTrue(
            result.Succeeded(), "process packet send failed: %s" % (result.GetError())
        )

        # Run a bit to give us a change to collect profile data:
        bkpt.SetIgnoreCount(1)
        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertEqual(len(threads), 1, "Hit our breakpoint again.")
        str = process.GetAsyncProfileData(1000)
        self.assertGreater(len(str), 0, "Got some profile data")

        # Now make the profiling interval very long and try to detach.
        interp.HandleCommand(
            "process plugin packet send 'QSetEnableAsyncProfiling;enable:1;interval_usec:10000000;scan_type=0x5;'",
            result,
        )
        self.assertTrue(
            result.Succeeded(), "process packet send failed: %s" % (result.GetError())
        )
        self.dbg.SetAsync(True)
        listener = self.dbg.GetListener()

        # We don't want to hit our breakpoint anymore.
        bkpt.SetEnabled(False)

        # Record our process pid so we can kill it since we are going to detach...
        self.pid = process.GetProcessID()

        def cleanup():
            self.dbg.SetAsync(False)
            os.kill(self.pid, signal.SIGKILL)

        self.addTearDownHook(cleanup)

        process.Continue()

        event = lldb.SBEvent()
        success = listener.WaitForEventForBroadcaster(
            0, process.GetBroadcaster(), event
        )
        self.assertTrue(success, "Got an event which should be running.")
        event_state = process.GetStateFromEvent(event)
        self.assertState(event_state, lldb.eStateRunning, "Got the running event")

        # Now detach:
        error = process.Detach()
        self.assertSuccess(error, "Detached successfully")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *

# This test checks that source code location is shown correctly
# when DWARF5 debug information is used.


class TestTargetSourceMap(TestBase):
    def test_source_map(self):
        # Set the target soure map to map "./" to the current test directory.
        yaml_path = os.path.join(self.getSourceDir(), "a.yaml")
        obj_path = self.getBuildArtifact("a.out")
        self.yaml2obj(yaml_path, obj_path)

        # Create a target with the object file we just created from YAML
        target = self.dbg.CreateTarget(obj_path)

        # Check we are able to show the locations properly.
        self.expect(
            "b main",
            VALID_BREAKPOINT_LOCATION,
            substrs=["main + 13 at test.cpp:2:3, address = 0x000000000040052d"],
        )

        self.expect(
            "b foo",
            VALID_BREAKPOINT_LOCATION,
            substrs=["foo() + 4 at test.cpp:6:1, address = 0x0000000000400534"],
        )



"""Test corefiles with "main bin spec"/"load binary" with only vmaddrs works."""


import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestMultipleBinaryCorefile(TestBase):
    def initial_setup(self):
        self.build()
        self.aout_exe_basename = "a.out"
        self.libone_exe_basename = "libone.dylib"
        self.libtwo_exe_basename = "libtwo.dylib"
        self.aout_exe = self.getBuildArtifact(self.aout_exe_basename)
        self.aout_slide = 0x5000
        self.libone_exe = self.getBuildArtifact(self.libone_exe_basename)
        self.libone_slide = 0x100840000
        self.libtwo_exe = self.getBuildArtifact(self.libtwo_exe_basename)
        self.libtwo_slide = 0
        self.corefile = self.getBuildArtifact("multiple-binaries.core")
        self.create_corefile = self.getBuildArtifact("create-multibin-corefile")
        cmd = "%s %s %s@%x %s@%x %s@%x" % (
            self.create_corefile,
            self.corefile,
            self.aout_exe,
            self.aout_slide,
            self.libone_exe,
            self.libone_slide,
            self.libtwo_exe,
            self.libtwo_slide,
        )
        if self.TraceOn():
            print("Creating corefile with command %s")
        call(cmd, shell=True)

    def load_corefile_and_test(self):
        target = self.dbg.CreateTarget("")
        err = lldb.SBError()
        if self.TraceOn():
            print("loading corefile %s" % self.corefile)
        process = target.LoadCore(self.corefile)
        self.assertTrue(process.IsValid())
        if self.TraceOn():
            print("image list after loading corefile:")
            self.runCmd("image list")

        ## We don't have libone.dylib in the global module cache or from
        ## dsymForUUID, and lldb will not read the binary out of memory.
        self.assertEqual(target.GetNumModules(), 2)
        fspec = target.GetModuleAtIndex(0).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), self.aout_exe_basename)

        fspec = target.GetModuleAtIndex(1).GetFileSpec()
        self.assertEqual(fspec.GetFilename(), self.libtwo_exe_basename)

        # Executables "always" have this base address
        aout_load = (
            target.GetModuleAtIndex(0)
            .GetObjectFileHeaderAddress()
            .GetLoadAddress(target)
        )
        self.assertEqual(aout_load, 0x100000000 + self.aout_slide)

        # Value from Makefile
        libtwo_load = (
            target.GetModuleAtIndex(1)
            .GetObjectFileHeaderAddress()
            .GetLoadAddress(target)
        )
        self.assertEqual(libtwo_load, self.libtwo_slide)

        self.dbg.DeleteTarget(target)
        self.dbg.Clear()

    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(archs=no_match(["x86_64", "arm64", "arm64e", "aarch64"]))
    @skipIfRemote
    @skipUnlessDarwin
    def test_corefile_binaries_dsymforuuid(self):
        self.initial_setup()

        if self.TraceOn():
            self.runCmd("log enable lldb dyld host")
            self.addTearDownHook(lambda: self.runCmd("log disable lldb dyld host"))

        ## We can hook in our dsym-for-uuid shell script to lldb with this env
        ## var instead of requiring a defaults write.
        dsym_for_uuid = self.getBuildArtifact("dsym-for-uuid.sh")
        os.environ["LLDB_APPLE_DSYMFORUUID_EXECUTABLE"] = dsym_for_uuid
        if self.TraceOn():
            print("Setting env var LLDB_APPLE_DSYMFORUUID_EXECUTABLE=" + dsym_for_uuid)
        self.addTearDownHook(
            lambda: os.environ.pop("LLDB_APPLE_DSYMFORUUID_EXECUTABLE", None)
        )

        self.runCmd("settings set target.load-script-from-symbol-file true")
        self.addTearDownHook(
            lambda: self.runCmd(
                "settings set target.load-script-from-symbol-file false"
            )
        )

        dwarfdump_uuid_regex = re.compile(r"UUID: ([-0-9a-fA-F]+) \(([^\(]+)\) .*")
        dwarfdump_cmd_output = subprocess.check_output(
            ('/usr/bin/dwarfdump --uuid "%s"' % self.libtwo_exe), shell=True
        ).decode("utf-8")
        libtwo_uuid = None
        for line in dwarfdump_cmd_output.splitlines():
            match = dwarfdump_uuid_regex.search(line)
            if match:
                libtwo_uuid = match.group(1)
        self.assertNotEqual(
            libtwo_uuid, None, "Could not get uuid of built libtwo.dylib"
        )
        dwarfdump_cmd_output = subprocess.check_output(
            ('/usr/bin/dwarfdump --uuid "%s"' % self.aout_exe), shell=True
        ).decode("utf-8")
        aout_uuid = None
        for line in dwarfdump_cmd_output.splitlines():
            match = dwarfdump_uuid_regex.search(line)
            if match:
                aout_uuid = match.group(1)
        self.assertNotEqual(aout_uuid, None, "Could not get uuid of built a.out")

        ###  Create our dsym-for-uuid shell script which returns aout_exe
        shell_cmds = [
            "#! /bin/sh",
            "# the last argument is the uuid",
            "while [ $# -gt 1 ]",
            "do",
            "  shift",
            "done",
            'echo "<?xml version=\\"1.0\\" encoding=\\"UTF-8\\"?>"',
            'echo "<!DOCTYPE plist PUBLIC \\"-//Apple//DTD PLIST 1.0//EN\\" \\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\\">"',
            'echo "<plist version=\\"1.0\\">"',
            'echo "  <dict>"',
            'echo "    <key>$1</key>"',
            'echo "    <dict>"',
            "",
            'if [ "$1" != "%s" -a "$1" != "%s" ]' % (libtwo_uuid, aout_uuid),
            "then",
            '  echo "      <key>DBGError</key>"',
            '  echo "      <string>not found by $0</string>"',
            '  echo "    </dict>"',
            '  echo "  </dict>"',
            '  echo "</plist>"',
            "  exit 0",
            "fi",
            #  UUID matches libtwo.dylib
            'if [ "$1" = "%s" ]' % (libtwo_uuid),
            "then",
            "  uuid=%s" % libtwo_uuid,
            "  bin=%s" % self.libtwo_exe,
            "  dsym=%s.dSYM/Contents/Resources/DWARF/%s"
            % (self.libtwo_exe, os.path.basename(self.libtwo_exe)),
            "fi",
            #  UUID matches a.out
            'if [ "$1" = "%s" ]' % (aout_uuid),
            "then",
            "  uuid=%s" % aout_uuid,
            "  bin=%s" % self.aout_exe,
            "  dsym=%s.dSYM/Contents/Resources/DWARF/%s"
            % (self.aout_exe, os.path.basename(self.aout_exe)),
            "fi",
            "",
            'echo "      <key>DBGDSYMPath</key>"',
            'echo "      <string>$dsym</string>"',
            'echo "      <key>DBGSymbolRichExecutable</key>"',
            'echo "      <string>$bin</string>"',
            'echo "    </dict>"',
            'echo "  </dict>"',
            'echo "</plist>"',
            "exit 0",
        ]

        with open(dsym_for_uuid, "w") as writer:
            for l in shell_cmds:
                writer.write(l + "\n")

        os.chmod(dsym_for_uuid, 0o755)

        # Register TWO of our binaries, but require dsymForUUID to find the third.
        target = self.dbg.CreateTarget(self.aout_exe, "", "", False, lldb.SBError())
        self.dbg.DeleteTarget(target)

        if self.TraceOn():
            print("Global image list, before loading corefile:")
            self.runCmd("image list -g")

        self.load_corefile_and_test()

    @skipIf(archs=no_match(["x86_64", "arm64", "arm64e", "aarch64"]))
    @skipIfRemote
    @skipUnlessDarwin
    def test_corefile_binaries_preloaded(self):
        self.initial_setup()

        if self.TraceOn():
            self.runCmd("log enable lldb dyld host")
            self.addTearDownHook(lambda: self.runCmd("log disable lldb dyld host"))

        # Register all three binaries in lldb's global module
        # cache, then throw the Targets away.
        target = self.dbg.CreateTarget(self.aout_exe, "", "", False, lldb.SBError())
        self.dbg.DeleteTarget(target)
        target = self.dbg.CreateTarget(self.libtwo_exe, "", "", False, lldb.SBError())
        self.dbg.DeleteTarget(target)

        if self.TraceOn():
            print("Global image list, before loading corefile:")
            self.runCmd("image list -g")

        self.load_corefile_and_test()



import os, struct, signal

from typing import Any, Dict

import lldb
from lldb.plugins.scripted_process import ScriptedProcess
from lldb.plugins.scripted_process import ScriptedThread
from lldb.plugins.scripted_process import ScriptedFrame


def my_python_function(x, y):
    """A sample Python function to demonstrate Python source display in scripted frames."""
    result = x + y
    if result > 100:
        return result * 2
    else:
        return result


class DummyStopHook:
    def __init__(self, target, args):
        self.target = target
        self.args = args

    def handle_stop(self, exe_ctx, stream):
        print("My DummyStopHook triggered. Printing args: \n%s" % self.args)
        sp = exe_ctx.process.GetScriptedImplementation()
        sp.handled_stop = True

class DummyScriptedProcess(ScriptedProcess):
    memory = None

    def __init__(self, exe_ctx: lldb.SBExecutionContext, args: lldb.SBStructuredData):
        super().__init__(exe_ctx, args)
        self.threads[0] = DummyScriptedThread(self, args)
        self.memory = {}
        addr = 0x500000000
        debugger = self.target.GetDebugger()
        index = debugger.GetIndexOfTarget(self.target)
        self.memory[addr] = "Hello, target " + str(index)
        self.handled_stop = False

    def read_memory_at_address(
        self, addr: int, size: int, error: lldb.SBError
    ) -> lldb.SBData:
        data = lldb.SBData().CreateDataFromCString(
            self.target.GetByteOrder(), self.target.GetCodeByteSize(), self.memory[addr]
        )

        return data

    def write_memory_at_address(self, addr, data, error):
        self.memory[addr] = data.GetString(error, 0)
        return len(self.memory[addr]) + 1

    def get_loaded_images(self):
        return self.loaded_images

    def get_process_id(self) -> int:
        return 42

    def should_stop(self) -> bool:
        return True

    def is_alive(self) -> bool:
        return True

    def get_scripted_thread_plugin(self):
        return DummyScriptedThread.__module__ + "." + DummyScriptedThread.__name__

    def my_super_secret_method(self):
        if hasattr(self, "my_super_secret_member"):
            return self.my_super_secret_member
        else:
            return None


class DummyScriptedThread(ScriptedThread):
    def __init__(self, process, args):
        super().__init__(process, args)
        self.frames.append({"pc": 0x0100001B00})
        self.frames.append(DummyScriptedFrame(self, args, len(self.frames), "baz123"))
        self.frames.append(DummyScriptedFrame(self, args, len(self.frames), "bar"))
        self.frames.append(DummyScriptedFrame(self, args, len(self.frames), "foo"))

        cwd = os.path.dirname(os.path.abspath(__file__))

        le = lldb.SBLineEntry()
        le.SetFileSpec(lldb.SBFileSpec(os.path.join(cwd, "baz.cpp"), True))
        le.SetLine(9)
        le.SetColumn(10)

        sym_ctx = lldb.SBSymbolContext()
        sym_ctx.SetLineEntry(le)

        self.frames.append(
            DummyScriptedFrame(self, args, len(self.frames), "baz", sym_ctx)
        )

        # Add a frame with Python source
        code = my_python_function.__code__
        lineno = code.co_firstlineno
        col_offset = getattr(
            code, "co_firstcol_offset", 0
        )  # Python ≥3.11 has column info
        py_le = lldb.SBLineEntry()
        py_le.SetFileSpec(lldb.SBFileSpec(__file__, True))
        py_le.SetLine(lineno)  # Line where my_python_function is defined
        py_le.SetColumn(col_offset)

        py_sym_ctx = lldb.SBSymbolContext()
        py_sym_ctx.SetLineEntry(py_le)

        self.frames.append(
            DummyScriptedFrame(
                self, args, len(self.frames), "my_python_function", py_sym_ctx
            )
        )

    def get_thread_id(self) -> int:
        return 0x19

    def get_name(self) -> str:
        return DummyScriptedThread.__name__ + ".thread-1"

    def get_state(self) -> int:
        return lldb.eStateStopped

    def get_stop_reason(self) -> Dict[str, Any]:
        return {"type": lldb.eStopReasonTrace, "data": {}}

    def get_register_context(self) -> str:
        return struct.pack(
            "21Q",
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            10,
            11,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19,
            20,
            21,
        )


class DummyScriptedFrame(ScriptedFrame):
    def __init__(self, thread, args, id, name, sym_ctx=None):
        super().__init__(thread, args)
        self.id = id
        self.name = name
        self.sym_ctx = sym_ctx

    def get_id(self):
        return self.id

    def get_function_name(self):
        return self.name

    def get_register_context(self) -> str:
        return struct.pack(
            "21Q",
            0x10001,
            0x10002,
            0x10003,
            0x10004,
            0x10005,
            0x10006,
            0x10007,
            0x10008,
            0x10009,
            0x100010,
            0x100011,
            0x100012,
            0x100013,
            0x100014,
            0x100015,
            0x100016,
            0x100017,
            0x100018,
            0x100019,
            0x100020,
            0x100021,
        )

    def get_symbol_context(self):
        def get_symbol_context_for_function(func_name):
            module = self.target.FindModule(self.target.GetExecutable())
            if not module.IsValid():
                return None

            sym_ctx_list = module.FindFunctions(func_name)
            if not sym_ctx_list.IsValid() or sym_ctx_list.GetSize() == 0:
                return None

            return sym_ctx_list.GetContextAtIndex(0)

        return (
            self.sym_ctx if self.sym_ctx else get_symbol_context_for_function(self.name)
        )

    def get_scripted_frame_plugin(self):
        return DummyScriptedFrame.__module__ + "." + DummyScriptedFrame.__name__


def __lldb_init_module(debugger, dict):
    # This is used when loading the script in an interactive debug session to
    # automatically, register the stop-hook and launch the scripted process.
    if not "SKIP_SCRIPTED_PROCESS_LAUNCH" in os.environ:
        debugger.HandleCommand(
            "target stop-hook add -k first -v 1 -k second -v 2 -P %s.%s"
            % (__name__, DummyStopHook.__name__)
        )
        debugger.HandleCommand(
            "process launch -C %s.%s" % (__name__, DummyScriptedProcess.__name__)
        )
    else:
        print(
            "Name of the class that will manage the scripted process: '%s.%s'"
            % (__name__, DummyScriptedProcess.__name__)
        )
        print(
            "Name of the class that will manage the stop-hook: '%s.%s'"
            % (__name__, DummyStopHook.__name__)
        )



"""
Test that objective-c method returning BOOL works correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class MethodReturningBOOLTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # We'll use the test method name as the exe_name.
        self.exe_name = self.testMethodName
        # Find the line number to break inside main().
        self.main_source = "main.m"
        self.line = line_number(self.main_source, "// Set breakpoint here.")

    def test_method_ret_BOOL(self):
        """Test that objective-c method returning BOOL works correctly."""
        d = {"EXE": self.exe_name}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)
        self.expect(
            "process status",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=[
                "stop reason = breakpoint",
                " at %s:%d" % (self.main_source, self.line),
            ],
        )

        # rdar://problem/9691614
        self.runCmd("expression (int)[my isValid]")



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetModule()
    obj.GetCompileUnit()
    obj.GetFunction()
    obj.GetBlock()
    obj.GetLineEntry()
    obj.GetSymbol()
    obj.GetDescription(lldb.SBStream())



import lldb
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestTraceEvents(TraceIntelPTTestCaseBase):
    @testSBAPIAndCommands
    def testCPUEvents(self):
        trace_description_file_path = os.path.join(
            self.getSourceDir(),
            "intelpt-multi-core-trace",
            "trace_missing_threads.json",
        )
        self.traceLoad(
            traceDescriptionFilePath=trace_description_file_path, substrs=["intel-pt"]
        )

        self.expect(
            "thread trace dump instructions 3 -e --forward -c 5",
            substrs=[
                """thread #3: tid = 3497496
    0: (event) HW clock tick [40450075477621505]
    1: (event) CPU core changed [new CPU=51]
    2: (event) HW clock tick [40450075477657246]
    3: (event) trace synchronization point [offset = 0x0x1331]
  m.out`foo() + 65 at multi_thread.cpp:12:21"""
            ],
        )

        self.expect(
            "thread trace dump instructions 3 -e --forward -c 5 -J",
            substrs=[
                """{
    "id": 0,
    "event": "HW clock tick",
    "hwClock": 40450075477621505
  },
  {
    "id": 1,
    "event": "CPU core changed",
    "cpuId": 51
  }"""
            ],
        )

    @skipIfNoIntelPT
    @testSBAPIAndCommands
    def testPauseEvents(self):
        """
        Everytime the target stops running on the CPU, a 'disabled' event will
        be emitted, which is represented by the TraceCursor API as a 'paused'
        event.
        """
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace-multi-file", "a.out")
        )
        self.expect("b 12")
        self.expect("r")
        self.traceStartThread()
        self.expect("n")
        self.expect("n")
        self.expect("si")
        self.expect("si")
        self.expect("si")
        # We ensure that the paused events are printed correctly forward
        self.expect(
            "thread trace dump instructions -e -f",
            patterns=[
                rf"""thread #1: tid = .*
    0: \(event\) trace synchronization point \[offset \= 0x0xec0\]
    1: \(event\) hardware disabled tracing
  a.out`main \+ 23 at main.cpp:12
    2: {ADDRESS_REGEX}    movl .*
    3: \(event\) software disabled tracing
    4: {ADDRESS_REGEX}    addl .*
    5: {ADDRESS_REGEX}    movl .*
    6: \(event\) software disabled tracing
  a.out`main \+ 34 \[inlined\] inline_function\(\) at main.cpp:4
    7: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 41 \[inlined\] inline_function\(\) \+ 7 at main.cpp:5
    8: {ADDRESS_REGEX}    movl .*
    9: {ADDRESS_REGEX}    addl .*
    10: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 52 \[inlined\] inline_function\(\) \+ 18 at main.cpp:6
    11: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 55 at main.cpp:14
    12: {ADDRESS_REGEX}    movl .*
    13: {ADDRESS_REGEX}    addl .*
    14: {ADDRESS_REGEX}    movl .*
    15: \(event\) software disabled tracing
  a.out`main \+ 63 at main.cpp:16
    16: {ADDRESS_REGEX}    callq  .* ; symbol stub for: foo\(\)
    17: \(event\) software disabled tracing
  a.out`symbol stub for: foo\(\)
    18: {ADDRESS_REGEX}    jmpq"""
            ],
        )

        # We ensure that the paused events are printed correctly backward
        self.expect(
            "thread trace dump instructions -e --id 18",
            patterns=[
                rf"""thread #1: tid = .*
  a.out`symbol stub for: foo\(\)
    18: {ADDRESS_REGEX}    jmpq .*
    17: \(event\) software disabled tracing
  a.out`main \+ 63 at main.cpp:16
    16: {ADDRESS_REGEX}    callq  .* ; symbol stub for: foo\(\)
    15: \(event\) software disabled tracing
  a.out`main \+ 60 at main.cpp:14
    14: {ADDRESS_REGEX}    movl .*
    13: {ADDRESS_REGEX}    addl .*
    12: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 52 \[inlined\] inline_function\(\) \+ 18 at main.cpp:6
    11: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 49 \[inlined\] inline_function\(\) \+ 15 at main.cpp:5
    10: {ADDRESS_REGEX}    movl .*
    9: {ADDRESS_REGEX}    addl .*
    8: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 34 \[inlined\] inline_function\(\) at main.cpp:4
    7: {ADDRESS_REGEX}    movl .*
    6: \(event\) software disabled tracing
  a.out`main \+ 31 at main.cpp:12
    5: {ADDRESS_REGEX}    movl .*
    4: {ADDRESS_REGEX}    addl .*
    3: \(event\) software disabled tracing
    2: {ADDRESS_REGEX}    movl .*
    1: \(event\) hardware disabled tracing
    0: \(event\) trace synchronization point \[offset \= 0x0xec0\]"""
            ],
        )



import os, json, struct, signal, uuid, tempfile

from typing import Any, Dict

import lldb
from lldb.plugins.scripted_process import ScriptedProcess
from lldb.plugins.scripted_process import ScriptedThread

from lldb.macosx.crashlog import CrashLog, CrashLogParser


class CrashLogScriptedProcess(ScriptedProcess):
    def parse_crashlog(self):
        if self.crashlog.process_id:
            if type(self.crashlog.process_id) is int:
                self.pid = self.crashlog.process_id
            elif type(self.crashlog.process_id) is str:
                self.pid = int(self.crashlog.process_id, 0)
            else:
                self.pid = super().get_process_id()
        self.addr_mask = self.crashlog.addr_mask
        self.crashed_thread_idx = self.crashlog.crashed_thread_idx
        self.loaded_images = []
        self.exception = self.crashlog.exception
        self.app_specific_thread = None
        if hasattr(self.crashlog, "asi"):
            self.metadata["asi"] = self.crashlog.asi
        if hasattr(self.crashlog, "asb"):
            self.extended_thread_info = self.crashlog.asb

        for thread in self.crashlog.threads:
            if (
                hasattr(thread, "app_specific_backtrace")
                and thread.app_specific_backtrace
            ):
                # We don't want to include the Application Specific Backtrace
                # Thread into the Scripted Process' Thread list.
                # Instead, we will try to extract the stackframe pcs from the
                # backtrace and inject that as the extended thread info.
                self.app_specific_thread = thread
                continue

            self.threads[thread.index] = CrashLogScriptedThread(self, None, thread)

        if self.app_specific_thread:
            self.extended_thread_info = CrashLogScriptedThread.resolve_stackframes(
                self.app_specific_thread, self.addr_mask, self.target
            )

    class CrashLogOptions:
        load_all_images = False
        crashed_only = True
        no_parallel_image_loading = False

    def __init__(self, exe_ctx: lldb.SBExecutionContext, args: lldb.SBStructuredData):
        super().__init__(exe_ctx, args)

        if not self.target or not self.target.IsValid():
            # Return error
            return

        self.crashlog_path = None

        crashlog_path = args.GetValueForKey("file_path")
        if crashlog_path and crashlog_path.IsValid():
            if crashlog_path.GetType() == lldb.eStructuredDataTypeString:
                self.crashlog_path = crashlog_path.GetStringValue(4096)

        if not self.crashlog_path:
            # Return error
            return

        self.options = self.CrashLogOptions()

        load_all_images = args.GetValueForKey("load_all_images")
        if load_all_images and load_all_images.IsValid():
            if load_all_images.GetType() == lldb.eStructuredDataTypeBoolean:
                self.options.load_all_images = load_all_images.GetBooleanValue()

        crashed_only = args.GetValueForKey("crashed_only")
        if crashed_only and crashed_only.IsValid():
            if crashed_only.GetType() == lldb.eStructuredDataTypeBoolean:
                self.options.crashed_only = crashed_only.GetBooleanValue()

        no_parallel_image_loading = args.GetValueForKey("no_parallel_image_loading")
        if no_parallel_image_loading and no_parallel_image_loading.IsValid():
            if no_parallel_image_loading.GetType() == lldb.eStructuredDataTypeBoolean:
                self.options.no_parallel_image_loading = (
                    no_parallel_image_loading.GetBooleanValue()
                )

        self.crashlog = None
        crashlog = args.GetValueForKey("crashlog")
        if crashlog and crashlog.IsValid():
            if crashlog.GetType() == lldb.eStructuredDataTypeGeneric:
                self.crashlog = crashlog.GetGenericValue()

        if not self.crashlog:
            # Return error
            return

        self.pid = super().get_process_id()
        self.crashed_thread_idx = 0
        self.exception = None
        self.extended_thread_info = None
        self.parse_crashlog()

    def read_memory_at_address(
        self, addr: int, size: int, error: lldb.SBError
    ) -> lldb.SBData:
        # NOTE: CrashLogs don't contain any memory.
        return lldb.SBData()

    def get_loaded_images(self):
        if len(self.loaded_images) == 0:
            self.crashlog.load_images(self.options, self.loaded_images)
        return self.loaded_images

    def should_stop(self) -> bool:
        return True

    def is_alive(self) -> bool:
        return True

    def get_scripted_thread_plugin(self):
        return CrashLogScriptedThread.__module__ + "." + CrashLogScriptedThread.__name__

    def get_process_metadata(self):
        return self.metadata


class CrashLogScriptedThread(ScriptedThread):
    def create_register_ctx(self):
        if not self.backing_thread or not len(self.backing_thread.registers):
            return dict.fromkeys(
                [*map(lambda reg: reg["name"], self.register_info["registers"])], 0
            )

        for reg in self.register_info["registers"]:
            reg_name = reg["name"]
            reg_alt_name = None
            if "alt-name" in reg:
                reg_alt_name = reg["alt-name"]
            if reg_name in self.backing_thread.registers:
                self.register_ctx[reg_name] = self.backing_thread.registers[reg_name]
            elif reg_alt_name and reg_alt_name in self.backing_thread.registers:
                self.register_ctx[reg_name] = self.backing_thread.registers[
                    reg_alt_name
                ]
            else:
                self.register_ctx[reg_name] = 0

        return self.register_ctx

    def resolve_stackframes(thread, addr_mask, target):
        frames = []
        for frame in thread.frames:
            frame_pc = frame.pc & addr_mask
            pc = frame_pc if frame.index == 0 or frame_pc == 0 else frame_pc - 1
            sym_addr = lldb.SBAddress()
            sym_addr.SetLoadAddress(pc, target)
            if not sym_addr.IsValid():
                continue
            frames.append({"idx": frame.index, "pc": pc})
        return frames

    def create_stackframes(self):
        if not (self.originating_process.options.load_all_images or self.has_crashed):
            return None

        if not self.backing_thread or not len(self.backing_thread.frames):
            return None

        self.frames = CrashLogScriptedThread.resolve_stackframes(
            self.backing_thread, self.originating_process.addr_mask, self.target
        )

        return self.frames

    def __init__(self, process, args, crashlog_thread):
        super().__init__(process, args)

        self.backing_thread = crashlog_thread
        self.idx = self.backing_thread.index
        self.tid = self.backing_thread.id
        self.name = self.backing_thread.name
        self.queue = self.backing_thread.queue
        self.has_crashed = self.originating_process.crashed_thread_idx == self.idx
        self.create_stackframes()

    def get_state(self):
        if not self.has_crashed:
            return lldb.eStateStopped
        return lldb.eStateCrashed

    def get_stop_reason(self) -> Dict[str, Any]:
        if not self.has_crashed:
            return {"type": lldb.eStopReasonNone}
        # TODO: Investigate what stop reason should be reported when crashed
        stop_reason = {"type": lldb.eStopReasonException, "data": {}}
        if self.originating_process.exception:
            stop_reason["data"]["mach_exception"] = self.originating_process.exception
        return stop_reason

    def get_register_context(self) -> str:
        if not self.register_ctx:
            self.register_ctx = self.create_register_ctx()

        return struct.pack(
            "{}Q".format(len(self.register_ctx)), *self.register_ctx.values()
        )

    def get_extended_info(self):
        if self.has_crashed:
            self.extended_info = self.originating_process.extended_thread_info
        return self.extended_info



"""
Test we can understand various layouts of the libc++'s std::string
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import functools


class LibcxxStringDataFormatterSimulatorTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def _run_test(self, defines):
        cxxflags_extras = " ".join(["-D%s" % d for d in defines])
        self.build(dictionary=dict(CXXFLAGS_EXTRAS=cxxflags_extras))
        lldbutil.run_to_source_breakpoint(
            self, "// Break here", lldb.SBFileSpec("main.cpp")
        )
        self.expect_var_path("shortstring", summary='"short"')
        self.expect_var_path("longstring", summary='"I am a very long string"')

        self.expect_expr("shortstring", result_summary='"short"')
        self.expect_expr("longstring", result_summary='"I am a very long string"')


for v in [None, "ALTERNATE_LAYOUT"]:
    for r in range(6):
        for c in range(5):
            name = "test_r%d_c%d" % (r, c)
            defines = ["REVISION=%d" % r, "COMPRESSED_PAIR_REV=%d" % c]
            if v:
                name += "_" + v
                defines += [v]

            @functools.wraps(LibcxxStringDataFormatterSimulatorTestCase._run_test)
            def test_method(self, defines=defines):
                LibcxxStringDataFormatterSimulatorTestCase._run_test(self, defines)

            setattr(LibcxxStringDataFormatterSimulatorTestCase, name, test_method)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCaseClassTemplateNonTypeParameterPack(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        self.expect_expr(
            "emptyNonTypePack",
            result_type="NonTypePack<>",
            result_children=[ValueCheck(name="a", type="int")],
        )
        self.expect_expr(
            "oneElemNonTypePack",
            result_type="NonTypePack<1>",
            result_children=[ValueCheck(name="a", type="int")],
        )
        self.expect_expr(
            "twoElemNonTypePack",
            result_type="NonTypePack<1, 2>",
            result_children=[ValueCheck(name="a", type="int")],
        )

        self.expect_expr(
            "emptyAnonNonTypePack",
            result_type="AnonNonTypePack<>",
            result_children=[ValueCheck(name="b", type="int")],
        )
        self.expect_expr(
            "oneElemAnonNonTypePack",
            result_type="AnonNonTypePack<1>",
            result_children=[ValueCheck(name="b", type="int")],
        )
        self.expect_expr(
            "twoElemAnonNonTypePack",
            result_type="AnonNonTypePack<1, 2>",
            result_children=[ValueCheck(name="b", type="int")],
        )

        self.expect_expr(
            "emptyAnonNonTypePackAfterTypeParam",
            result_type="AnonNonTypePackAfterTypeParam<int>",
            result_children=[ValueCheck(name="c", type="int")],
        )
        self.expect_expr(
            "oneElemAnonNonTypePackAfterTypeParam",
            result_type="AnonNonTypePackAfterTypeParam<int, 1>",
            result_children=[ValueCheck(name="c", type="int")],
        )

        self.expect_expr(
            "emptyAnonNonTypePackAfterAnonTypeParam",
            result_type="AnonNonTypePackAfterAnonTypeParam<int>",
            result_children=[ValueCheck(name="d", type="float")],
        )
        self.expect_expr(
            "oneElemAnonNonTypePackAfterAnonTypeParam",
            result_type="AnonNonTypePackAfterAnonTypeParam<int, 1>",
            result_children=[ValueCheck(name="d", type="float")],
        )

        self.expect_expr(
            "emptyNonTypePackAfterAnonTypeParam",
            result_type="NonTypePackAfterAnonTypeParam<int>",
            result_children=[ValueCheck(name="e", type="int")],
        )
        self.expect_expr(
            "oneElemNonTypePackAfterAnonTypeParam",
            result_type="NonTypePackAfterAnonTypeParam<int, 1>",
            result_children=[ValueCheck(name="e", type="int")],
        )

        self.expect_expr(
            "emptyNonTypePackAfterTypeParam",
            result_type="NonTypePackAfterTypeParam<int>",
            result_children=[ValueCheck(name="f", type="int")],
        )
        self.expect_expr(
            "oneElemNonTypePackAfterTypeParam",
            result_type="NonTypePackAfterTypeParam<int, 1>",
            result_children=[ValueCheck(name="f", type="int")],
        )

        self.expect_expr(
            "emptyAnonNonTypePackAfterNonTypeParam",
            result_type="AnonNonTypePackAfterNonTypeParam<1>",
            result_children=[ValueCheck(name="g", type="int")],
        )
        self.expect_expr(
            "oneElemAnonNonTypePackAfterNonTypeParam",
            result_type="AnonNonTypePackAfterNonTypeParam<1, 2>",
            result_children=[ValueCheck(name="g", type="int")],
        )

        self.expect_expr(
            "emptyAnonNonTypePackAfterAnonNonTypeParam",
            result_type="AnonNonTypePackAfterAnonNonTypeParam<1>",
            result_children=[ValueCheck(name="h", type="float")],
        )
        self.expect_expr(
            "oneElemAnonNonTypePackAfterAnonNonTypeParam",
            result_type="AnonNonTypePackAfterAnonNonTypeParam<1, 2>",
            result_children=[ValueCheck(name="h", type="float")],
        )

        self.expect_expr(
            "emptyNonTypePackAfterAnonNonTypeParam",
            result_type="NonTypePackAfterAnonNonTypeParam<1>",
            result_children=[ValueCheck(name="i", type="int")],
        )
        self.expect_expr(
            "oneElemNonTypePackAfterAnonNonTypeParam",
            result_type="NonTypePackAfterAnonNonTypeParam<1, 2>",
            result_children=[ValueCheck(name="i", type="int")],
        )

        self.expect_expr(
            "emptyNonTypePackAfterNonTypeParam",
            result_type="NonTypePackAfterNonTypeParam<1>",
            result_children=[ValueCheck(name="j", type="int")],
        )
        self.expect_expr(
            "oneElemNonTypePackAfterNonTypeParam",
            result_type="NonTypePackAfterNonTypeParam<1, 2>",
            result_children=[ValueCheck(name="j", type="int")],
        )



"""
Test that we return only the requested template instantiation.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class UniqueTypesTestCase3(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_test(self, debug_flags):
        """Test that we display the correct template instantiation."""
        self.build(dictionary=debug_flags)
        lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here", lldb.SBFileSpec("a.cpp")
        )
        self.expect_expr("a", result_type="S<int>")

    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler_version=["<", "15.0"])
    def test_simple_template_names(self):
        # Can't directly set CFLAGS_EXTRAS here because the Makefile can't
        # override an environment variable.
        self.do_test(dict(TEST_CFLAGS_EXTRAS="-gsimple-template-names"))

    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler_version=["<", "15.0"])
    def test_no_simple_template_names(self):
        self.do_test(dict(CFLAGS_EXTRAS="-gno-simple-template-names"))



"""
This is a sanity check that verifies that `repr(sbobject)` and `str(sbobject)`
produce the same string.
"""


import lldb
from lldbsuite.test.lldbtest import *


class TestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.assertEqual(repr(self.dbg), str(self.dbg))



"""
Test std::forward_list functionality with a decl from debug info as content.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDbgInfoContentForwardList(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(macos_version=["<", "15.0"])
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            list_type = "std::forward_list<Foo>"
        else:
            list_type = "std::forward_list<Foo, std::allocator<Foo> >"

        value_type = "value_type"

        # FIXME: This has three elements in it but the formatter seems to
        # calculate the wrong size and contents.
        self.expect_expr("a", result_type=list_type, result_summary="size=1")
        self.expect_expr("std::distance(a.begin(), a.end())", result_value="3")
        self.expect_expr("a.front().a", result_type="int", result_value="3")
        self.expect_expr("a.begin()->a", result_type="int", result_value="3")
